from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.vehicle_section_type import VehicleSectionType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.arrive_action import ArriveAction
    from ..models.base_action import BaseAction
    from ..models.charging_action import ChargingAction
    from ..models.charging_setup_action import ChargingSetupAction
    from ..models.continue_action import ContinueAction
    from ..models.deboard_action import DeboardAction
    from ..models.depart_action import DepartAction
    from ..models.exit_action import ExitAction
    from ..models.passthrough import Passthrough
    from ..models.ref_replacements import RefReplacements
    from ..models.routing_zone import RoutingZone
    from ..models.toll_cost import TollCost
    from ..models.toll_system import TollSystem
    from ..models.traffic_incident import TrafficIncident
    from ..models.turn_action import TurnAction
    from ..models.vehicle_departure import VehicleDeparture
    from ..models.vehicle_notice import VehicleNotice
    from ..models.vehicle_restriction import VehicleRestriction
    from ..models.vehicle_span import VehicleSpan
    from ..models.vehicle_summary import VehicleSummary
    from ..models.vehicle_transport import VehicleTransport
    from ..models.vehicle_travel_summary import VehicleTravelSummary
    from ..models.wait_action import WaitAction


T = TypeVar("T", bound="VehicleSection")


@_attrs_define
class VehicleSection:
    """Represent a section of a route

    Attributes:
        id (str): Unique identifier of the section
        type_ (VehicleSectionType): Section type used by the client to identify what extension to the BaseSection are
            available.
        departure (VehicleDeparture): Describe a departure or arrival location and time.
        arrival (VehicleDeparture): Describe a departure or arrival location and time.
        transport (VehicleTransport): Information about a transport
        pre_actions (list[BaseAction] | Unset): Actions that must be done prior to `departure`.
        actions (list[ArriveAction | ContinueAction | DepartAction | ExitAction | TurnAction] | Unset): Actions to be
            performed at or during a specific portion of a section.

            Action offsets represent the coordinate index in the polyline.

            *NOTE:* The following attributes are valid for turnByTurnActions only and are not populated for actions
              * currentRoad
              * nextRoad
              * exitSign
              * signPost
              * intersectionName
              * turnAngle
        language (str | Unset): Language of the localized strings in the section, if any, in BCP47 format.
        post_actions (list[ChargingAction | ChargingSetupAction | DeboardAction | WaitAction] | Unset): Actions that
            must be done after `arrival`.
        turn_by_turn_actions (list[ArriveAction | ContinueAction | DepartAction | ExitAction | TurnAction] | Unset):
            Turn-by-turn guidance actions.

            Action offsets represent the coordinate index in the polyline.
        passthrough (list[Passthrough] | Unset): List of via waypoints this section is passing through.

            Each via waypoint of the request that is a `passThrough=true` waypoint, appears as a
            `Passthrough` in the response. It appears in the section that starts with the closest
            non-passthrough via specified before it or origin.

            The passthrough vias appear in this list in the order they are traversed. They are
            traversed in the order they are specified in the request.
        summary (VehicleSummary | Unset): Total value of key attributes for a route section.
        travel_summary (VehicleTravelSummary | Unset): Total value of key attributes for a route section.
        polyline (str | Unset): Line string in [Flexible Polyline](https://github.com/heremaps/flexible-polyline)
            format. Coordinates are in the WGS84 coordinate system, including `Elevation` (if present). Example:
            A05xgKuy2xCx9B7vUl0OhnR54EqSzpEl-HxjD3pBiGnyGi2CvwFsgD3nD4vB6e.
        notices (list[VehicleNotice] | Unset): Contains a list of issues related to this section of the route.

            Notices must be carefully evaluated and, if deemed necessary, the route section should be discarded accordingly.
            In particular, the user should be aware that new notice codes may be added at any time. If an unrecognized
            notice code appears with a `critical` severity level, the route section must be discarded.
            Please refer to the `code` attribute for possible values.
        spans (list[VehicleSpan] | Unset): Spans attached to a `Section` describing vehicle content.
        routing_zones (list[RoutingZone] | Unset): A list of routing zones that are applicable to the section.

            Elements of this list will be referenced by indexes within the `span` attribute of the section.
        truck_road_types (list[str] | Unset): A list of truck road types that are applicable to the section.

            Elements of this list will be referenced by indexes within the `span` attribute of the section.

            A truck road type is an identifier associated with roads that have additional regulations applied by local
            administration for
            traversal by heavy vehicles like trucks. For example, the BK Bearing Class regulations in Sweden, and ET
            categories in Mexico.
            The identifiers of supported truck road types are specified at HERE Map Content
            [TruckRoadType](https://www.here.com/docs/bundle/map-content-schema-data-
            specification/page/topics_schema/truckroadtypeattribute.html).

            These names should be provided when avoiding a certain truck road type.
        incidents (list[TrafficIncident] | Unset): A list of all incidents that apply to the section.
        ref_replacements (RefReplacements | Unset): Dictionary of placeholders to replacement strings for the compact
            representation of map entity references.
        toll_systems (list[TollSystem] | Unset): An array of toll authorities that collect payments for the use of (part
            of) the specified section of the route.
        tolls (list[TollCost] | Unset): Detail of tolls to be paid for traversing the specified section.
        consumption_type (str | Unset): Extensible enum: `electric` `diesel` `petrol` `lpg` `cng` `lng` `ethanol`
            `propane` `hydrogen` `...`
            Vehicle consumption type. The energy or fuel consumption type (electric, diesel, petrol, lpg, cng, etc.)
            of a vehicle which provides the relevant information required to parse (unit, etc.) the energy or fuel
            `consumption` value of a vehicle.
        no_through_restrictions (list[VehicleRestriction] | Unset): A list of all rules that restrict movement through
            the area without there being an
            origin, destination, or `via` waypoint inside of it. Only restrictions applicable
            to the vehicle for which the route was requested are returned.
    """

    id: str
    type_: VehicleSectionType
    departure: VehicleDeparture
    arrival: VehicleDeparture
    transport: VehicleTransport
    pre_actions: list[BaseAction] | Unset = UNSET
    actions: (
        list[ArriveAction | ContinueAction | DepartAction | ExitAction | TurnAction]
        | Unset
    ) = UNSET
    language: str | Unset = UNSET
    post_actions: (
        list[ChargingAction | ChargingSetupAction | DeboardAction | WaitAction] | Unset
    ) = UNSET
    turn_by_turn_actions: (
        list[ArriveAction | ContinueAction | DepartAction | ExitAction | TurnAction]
        | Unset
    ) = UNSET
    passthrough: list[Passthrough] | Unset = UNSET
    summary: VehicleSummary | Unset = UNSET
    travel_summary: VehicleTravelSummary | Unset = UNSET
    polyline: str | Unset = UNSET
    notices: list[VehicleNotice] | Unset = UNSET
    spans: list[VehicleSpan] | Unset = UNSET
    routing_zones: list[RoutingZone] | Unset = UNSET
    truck_road_types: list[str] | Unset = UNSET
    incidents: list[TrafficIncident] | Unset = UNSET
    ref_replacements: RefReplacements | Unset = UNSET
    toll_systems: list[TollSystem] | Unset = UNSET
    tolls: list[TollCost] | Unset = UNSET
    consumption_type: str | Unset = UNSET
    no_through_restrictions: list[VehicleRestriction] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.arrive_action import ArriveAction
        from ..models.charging_action import ChargingAction
        from ..models.charging_setup_action import ChargingSetupAction
        from ..models.continue_action import ContinueAction
        from ..models.deboard_action import DeboardAction
        from ..models.depart_action import DepartAction
        from ..models.exit_action import ExitAction

        id = self.id

        type_ = self.type_.value

        departure = self.departure.to_dict()

        arrival = self.arrival.to_dict()

        transport = self.transport.to_dict()

        pre_actions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.pre_actions, Unset):
            pre_actions = []
            for pre_actions_item_data in self.pre_actions:
                pre_actions_item = pre_actions_item_data.to_dict()
                pre_actions.append(pre_actions_item)

        actions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.actions, Unset):
            actions = []
            for actions_item_data in self.actions:
                actions_item: dict[str, Any]
                if isinstance(actions_item_data, DepartAction):
                    actions_item = actions_item_data.to_dict()
                elif isinstance(actions_item_data, ArriveAction):
                    actions_item = actions_item_data.to_dict()
                elif isinstance(actions_item_data, ContinueAction):
                    actions_item = actions_item_data.to_dict()
                elif isinstance(actions_item_data, ExitAction):
                    actions_item = actions_item_data.to_dict()
                else:
                    actions_item = actions_item_data.to_dict()

                actions.append(actions_item)

        language = self.language

        post_actions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.post_actions, Unset):
            post_actions = []
            for post_actions_item_data in self.post_actions:
                post_actions_item: dict[str, Any]
                if isinstance(post_actions_item_data, ChargingSetupAction):
                    post_actions_item = post_actions_item_data.to_dict()
                elif isinstance(post_actions_item_data, ChargingAction):
                    post_actions_item = post_actions_item_data.to_dict()
                elif isinstance(post_actions_item_data, DeboardAction):
                    post_actions_item = post_actions_item_data.to_dict()
                else:
                    post_actions_item = post_actions_item_data.to_dict()

                post_actions.append(post_actions_item)

        turn_by_turn_actions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.turn_by_turn_actions, Unset):
            turn_by_turn_actions = []
            for turn_by_turn_actions_item_data in self.turn_by_turn_actions:
                turn_by_turn_actions_item: dict[str, Any]
                if isinstance(turn_by_turn_actions_item_data, DepartAction):
                    turn_by_turn_actions_item = turn_by_turn_actions_item_data.to_dict()
                elif isinstance(turn_by_turn_actions_item_data, ArriveAction):
                    turn_by_turn_actions_item = turn_by_turn_actions_item_data.to_dict()
                elif isinstance(turn_by_turn_actions_item_data, ContinueAction):
                    turn_by_turn_actions_item = turn_by_turn_actions_item_data.to_dict()
                elif isinstance(turn_by_turn_actions_item_data, ExitAction):
                    turn_by_turn_actions_item = turn_by_turn_actions_item_data.to_dict()
                else:
                    turn_by_turn_actions_item = turn_by_turn_actions_item_data.to_dict()

                turn_by_turn_actions.append(turn_by_turn_actions_item)

        passthrough: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.passthrough, Unset):
            passthrough = []
            for passthrough_item_data in self.passthrough:
                passthrough_item = passthrough_item_data.to_dict()
                passthrough.append(passthrough_item)

        summary: dict[str, Any] | Unset = UNSET
        if not isinstance(self.summary, Unset):
            summary = self.summary.to_dict()

        travel_summary: dict[str, Any] | Unset = UNSET
        if not isinstance(self.travel_summary, Unset):
            travel_summary = self.travel_summary.to_dict()

        polyline = self.polyline

        notices: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.notices, Unset):
            notices = []
            for notices_item_data in self.notices:
                notices_item = notices_item_data.to_dict()
                notices.append(notices_item)

        spans: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.spans, Unset):
            spans = []
            for spans_item_data in self.spans:
                spans_item = spans_item_data.to_dict()
                spans.append(spans_item)

        routing_zones: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.routing_zones, Unset):
            routing_zones = []
            for routing_zones_item_data in self.routing_zones:
                routing_zones_item = routing_zones_item_data.to_dict()
                routing_zones.append(routing_zones_item)

        truck_road_types: list[str] | Unset = UNSET
        if not isinstance(self.truck_road_types, Unset):
            truck_road_types = self.truck_road_types

        incidents: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.incidents, Unset):
            incidents = []
            for incidents_item_data in self.incidents:
                incidents_item = incidents_item_data.to_dict()
                incidents.append(incidents_item)

        ref_replacements: dict[str, Any] | Unset = UNSET
        if not isinstance(self.ref_replacements, Unset):
            ref_replacements = self.ref_replacements.to_dict()

        toll_systems: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.toll_systems, Unset):
            toll_systems = []
            for toll_systems_item_data in self.toll_systems:
                toll_systems_item = toll_systems_item_data.to_dict()
                toll_systems.append(toll_systems_item)

        tolls: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.tolls, Unset):
            tolls = []
            for tolls_item_data in self.tolls:
                tolls_item = tolls_item_data.to_dict()
                tolls.append(tolls_item)

        consumption_type = self.consumption_type

        no_through_restrictions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.no_through_restrictions, Unset):
            no_through_restrictions = []
            for no_through_restrictions_item_data in self.no_through_restrictions:
                no_through_restrictions_item = (
                    no_through_restrictions_item_data.to_dict()
                )
                no_through_restrictions.append(no_through_restrictions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "type": type_,
            "departure": departure,
            "arrival": arrival,
            "transport": transport,
        })
        if pre_actions is not UNSET:
            field_dict["preActions"] = pre_actions
        if actions is not UNSET:
            field_dict["actions"] = actions
        if language is not UNSET:
            field_dict["language"] = language
        if post_actions is not UNSET:
            field_dict["postActions"] = post_actions
        if turn_by_turn_actions is not UNSET:
            field_dict["turnByTurnActions"] = turn_by_turn_actions
        if passthrough is not UNSET:
            field_dict["passthrough"] = passthrough
        if summary is not UNSET:
            field_dict["summary"] = summary
        if travel_summary is not UNSET:
            field_dict["travelSummary"] = travel_summary
        if polyline is not UNSET:
            field_dict["polyline"] = polyline
        if notices is not UNSET:
            field_dict["notices"] = notices
        if spans is not UNSET:
            field_dict["spans"] = spans
        if routing_zones is not UNSET:
            field_dict["routingZones"] = routing_zones
        if truck_road_types is not UNSET:
            field_dict["truckRoadTypes"] = truck_road_types
        if incidents is not UNSET:
            field_dict["incidents"] = incidents
        if ref_replacements is not UNSET:
            field_dict["refReplacements"] = ref_replacements
        if toll_systems is not UNSET:
            field_dict["tollSystems"] = toll_systems
        if tolls is not UNSET:
            field_dict["tolls"] = tolls
        if consumption_type is not UNSET:
            field_dict["consumptionType"] = consumption_type
        if no_through_restrictions is not UNSET:
            field_dict["noThroughRestrictions"] = no_through_restrictions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.arrive_action import ArriveAction
        from ..models.base_action import BaseAction
        from ..models.charging_action import ChargingAction
        from ..models.charging_setup_action import ChargingSetupAction
        from ..models.continue_action import ContinueAction
        from ..models.deboard_action import DeboardAction
        from ..models.depart_action import DepartAction
        from ..models.exit_action import ExitAction
        from ..models.passthrough import Passthrough
        from ..models.ref_replacements import RefReplacements
        from ..models.routing_zone import RoutingZone
        from ..models.toll_cost import TollCost
        from ..models.toll_system import TollSystem
        from ..models.traffic_incident import TrafficIncident
        from ..models.turn_action import TurnAction
        from ..models.vehicle_departure import VehicleDeparture
        from ..models.vehicle_notice import VehicleNotice
        from ..models.vehicle_restriction import VehicleRestriction
        from ..models.vehicle_span import VehicleSpan
        from ..models.vehicle_summary import VehicleSummary
        from ..models.vehicle_transport import VehicleTransport
        from ..models.vehicle_travel_summary import VehicleTravelSummary
        from ..models.wait_action import WaitAction

        d = dict(src_dict)
        id = d.pop("id")

        type_ = VehicleSectionType(d.pop("type"))

        departure = VehicleDeparture.from_dict(d.pop("departure"))

        arrival = VehicleDeparture.from_dict(d.pop("arrival"))

        transport = VehicleTransport.from_dict(d.pop("transport"))

        _pre_actions = d.pop("preActions", UNSET)
        pre_actions: list[BaseAction] | Unset = UNSET
        if _pre_actions is not UNSET:
            pre_actions = []
            for pre_actions_item_data in _pre_actions:
                pre_actions_item = BaseAction.from_dict(pre_actions_item_data)

                pre_actions.append(pre_actions_item)

        _actions = d.pop("actions", UNSET)
        actions: (
            list[ArriveAction | ContinueAction | DepartAction | ExitAction | TurnAction]
            | Unset
        ) = UNSET
        if _actions is not UNSET:
            actions = []
            for actions_item_data in _actions:

                def _parse_actions_item(
                    data: object,
                ) -> (
                    ArriveAction
                    | ContinueAction
                    | DepartAction
                    | ExitAction
                    | TurnAction
                ):
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_action_type_0 = (
                            DepartAction.from_dict(data)
                        )

                        return componentsschemas_vehicle_action_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_action_type_1 = (
                            ArriveAction.from_dict(data)
                        )

                        return componentsschemas_vehicle_action_type_1
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_action_type_2 = (
                            ContinueAction.from_dict(data)
                        )

                        return componentsschemas_vehicle_action_type_2
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_action_type_3 = ExitAction.from_dict(
                            data
                        )

                        return componentsschemas_vehicle_action_type_3
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_vehicle_action_type_4 = TurnAction.from_dict(data)

                    return componentsschemas_vehicle_action_type_4

                actions_item = _parse_actions_item(actions_item_data)

                actions.append(actions_item)

        language = d.pop("language", UNSET)

        _post_actions = d.pop("postActions", UNSET)
        post_actions: (
            list[ChargingAction | ChargingSetupAction | DeboardAction | WaitAction]
            | Unset
        ) = UNSET
        if _post_actions is not UNSET:
            post_actions = []
            for post_actions_item_data in _post_actions:

                def _parse_post_actions_item(
                    data: object,
                ) -> ChargingAction | ChargingSetupAction | DeboardAction | WaitAction:
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_post_action_type_0 = (
                            ChargingSetupAction.from_dict(data)
                        )

                        return componentsschemas_vehicle_post_action_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_post_action_type_1 = (
                            ChargingAction.from_dict(data)
                        )

                        return componentsschemas_vehicle_post_action_type_1
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_post_action_type_2 = (
                            DeboardAction.from_dict(data)
                        )

                        return componentsschemas_vehicle_post_action_type_2
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_vehicle_post_action_type_3 = WaitAction.from_dict(
                        data
                    )

                    return componentsschemas_vehicle_post_action_type_3

                post_actions_item = _parse_post_actions_item(post_actions_item_data)

                post_actions.append(post_actions_item)

        _turn_by_turn_actions = d.pop("turnByTurnActions", UNSET)
        turn_by_turn_actions: (
            list[ArriveAction | ContinueAction | DepartAction | ExitAction | TurnAction]
            | Unset
        ) = UNSET
        if _turn_by_turn_actions is not UNSET:
            turn_by_turn_actions = []
            for turn_by_turn_actions_item_data in _turn_by_turn_actions:

                def _parse_turn_by_turn_actions_item(
                    data: object,
                ) -> (
                    ArriveAction
                    | ContinueAction
                    | DepartAction
                    | ExitAction
                    | TurnAction
                ):
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_action_type_0 = (
                            DepartAction.from_dict(data)
                        )

                        return componentsschemas_vehicle_action_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_action_type_1 = (
                            ArriveAction.from_dict(data)
                        )

                        return componentsschemas_vehicle_action_type_1
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_action_type_2 = (
                            ContinueAction.from_dict(data)
                        )

                        return componentsschemas_vehicle_action_type_2
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_action_type_3 = ExitAction.from_dict(
                            data
                        )

                        return componentsschemas_vehicle_action_type_3
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_vehicle_action_type_4 = TurnAction.from_dict(data)

                    return componentsschemas_vehicle_action_type_4

                turn_by_turn_actions_item = _parse_turn_by_turn_actions_item(
                    turn_by_turn_actions_item_data
                )

                turn_by_turn_actions.append(turn_by_turn_actions_item)

        _passthrough = d.pop("passthrough", UNSET)
        passthrough: list[Passthrough] | Unset = UNSET
        if _passthrough is not UNSET:
            passthrough = []
            for passthrough_item_data in _passthrough:
                passthrough_item = Passthrough.from_dict(passthrough_item_data)

                passthrough.append(passthrough_item)

        _summary = d.pop("summary", UNSET)
        summary: VehicleSummary | Unset
        if isinstance(_summary, Unset):
            summary = UNSET
        else:
            summary = VehicleSummary.from_dict(_summary)

        _travel_summary = d.pop("travelSummary", UNSET)
        travel_summary: VehicleTravelSummary | Unset
        if isinstance(_travel_summary, Unset):
            travel_summary = UNSET
        else:
            travel_summary = VehicleTravelSummary.from_dict(_travel_summary)

        polyline = d.pop("polyline", UNSET)

        _notices = d.pop("notices", UNSET)
        notices: list[VehicleNotice] | Unset = UNSET
        if _notices is not UNSET:
            notices = []
            for notices_item_data in _notices:
                notices_item = VehicleNotice.from_dict(notices_item_data)

                notices.append(notices_item)

        _spans = d.pop("spans", UNSET)
        spans: list[VehicleSpan] | Unset = UNSET
        if _spans is not UNSET:
            spans = []
            for spans_item_data in _spans:
                spans_item = VehicleSpan.from_dict(spans_item_data)

                spans.append(spans_item)

        _routing_zones = d.pop("routingZones", UNSET)
        routing_zones: list[RoutingZone] | Unset = UNSET
        if _routing_zones is not UNSET:
            routing_zones = []
            for routing_zones_item_data in _routing_zones:
                routing_zones_item = RoutingZone.from_dict(routing_zones_item_data)

                routing_zones.append(routing_zones_item)

        truck_road_types = cast(list[str], d.pop("truckRoadTypes", UNSET))

        _incidents = d.pop("incidents", UNSET)
        incidents: list[TrafficIncident] | Unset = UNSET
        if _incidents is not UNSET:
            incidents = []
            for incidents_item_data in _incidents:
                incidents_item = TrafficIncident.from_dict(incidents_item_data)

                incidents.append(incidents_item)

        _ref_replacements = d.pop("refReplacements", UNSET)
        ref_replacements: RefReplacements | Unset
        if isinstance(_ref_replacements, Unset):
            ref_replacements = UNSET
        else:
            ref_replacements = RefReplacements.from_dict(_ref_replacements)

        _toll_systems = d.pop("tollSystems", UNSET)
        toll_systems: list[TollSystem] | Unset = UNSET
        if _toll_systems is not UNSET:
            toll_systems = []
            for toll_systems_item_data in _toll_systems:
                toll_systems_item = TollSystem.from_dict(toll_systems_item_data)

                toll_systems.append(toll_systems_item)

        _tolls = d.pop("tolls", UNSET)
        tolls: list[TollCost] | Unset = UNSET
        if _tolls is not UNSET:
            tolls = []
            for tolls_item_data in _tolls:
                tolls_item = TollCost.from_dict(tolls_item_data)

                tolls.append(tolls_item)

        consumption_type = d.pop("consumptionType", UNSET)

        _no_through_restrictions = d.pop("noThroughRestrictions", UNSET)
        no_through_restrictions: list[VehicleRestriction] | Unset = UNSET
        if _no_through_restrictions is not UNSET:
            no_through_restrictions = []
            for no_through_restrictions_item_data in _no_through_restrictions:
                no_through_restrictions_item = VehicleRestriction.from_dict(
                    no_through_restrictions_item_data
                )

                no_through_restrictions.append(no_through_restrictions_item)

        vehicle_section = cls(
            id=id,
            type_=type_,
            departure=departure,
            arrival=arrival,
            transport=transport,
            pre_actions=pre_actions,
            actions=actions,
            language=language,
            post_actions=post_actions,
            turn_by_turn_actions=turn_by_turn_actions,
            passthrough=passthrough,
            summary=summary,
            travel_summary=travel_summary,
            polyline=polyline,
            notices=notices,
            spans=spans,
            routing_zones=routing_zones,
            truck_road_types=truck_road_types,
            incidents=incidents,
            ref_replacements=ref_replacements,
            toll_systems=toll_systems,
            tolls=tolls,
            consumption_type=consumption_type,
            no_through_restrictions=no_through_restrictions,
        )

        vehicle_section.additional_properties = d
        return vehicle_section

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.match_info import MatchInfo


T = TypeVar("T", bound="Parsing")


@_attrs_define
class Parsing:
    """
    Attributes:
        place_name (list[MatchInfo] | Unset): Place name matches
        country (list[MatchInfo] | Unset): Country matches
        state (list[MatchInfo] | Unset): State matches
        county (list[MatchInfo] | Unset): County matches
        city (list[MatchInfo] | Unset): City matches
        district (list[MatchInfo] | Unset): District matches
        subdistrict (list[MatchInfo] | Unset): Subdistrict matches
        street (list[MatchInfo] | Unset): Street matches
        block (list[MatchInfo] | Unset): Block matches
        subblock (list[MatchInfo] | Unset): Subblock matches
        house_number (list[MatchInfo] | Unset): HouseNumber matches
        postal_code (list[MatchInfo] | Unset): PostalCode matches
        building (list[MatchInfo] | Unset): Building matches
        secondary_units (list[MatchInfo] | Unset): secondaryUnits matches
        additional_info (list[MatchInfo] | Unset): Additional information extracted from the query that do not
            correspond to any recognized address components in the map data.
    """

    place_name: list[MatchInfo] | Unset = UNSET
    country: list[MatchInfo] | Unset = UNSET
    state: list[MatchInfo] | Unset = UNSET
    county: list[MatchInfo] | Unset = UNSET
    city: list[MatchInfo] | Unset = UNSET
    district: list[MatchInfo] | Unset = UNSET
    subdistrict: list[MatchInfo] | Unset = UNSET
    street: list[MatchInfo] | Unset = UNSET
    block: list[MatchInfo] | Unset = UNSET
    subblock: list[MatchInfo] | Unset = UNSET
    house_number: list[MatchInfo] | Unset = UNSET
    postal_code: list[MatchInfo] | Unset = UNSET
    building: list[MatchInfo] | Unset = UNSET
    secondary_units: list[MatchInfo] | Unset = UNSET
    additional_info: list[MatchInfo] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        place_name: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.place_name, Unset):
            place_name = []
            for place_name_item_data in self.place_name:
                place_name_item = place_name_item_data.to_dict()
                place_name.append(place_name_item)

        country: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.country, Unset):
            country = []
            for country_item_data in self.country:
                country_item = country_item_data.to_dict()
                country.append(country_item)

        state: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.state, Unset):
            state = []
            for state_item_data in self.state:
                state_item = state_item_data.to_dict()
                state.append(state_item)

        county: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.county, Unset):
            county = []
            for county_item_data in self.county:
                county_item = county_item_data.to_dict()
                county.append(county_item)

        city: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.city, Unset):
            city = []
            for city_item_data in self.city:
                city_item = city_item_data.to_dict()
                city.append(city_item)

        district: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.district, Unset):
            district = []
            for district_item_data in self.district:
                district_item = district_item_data.to_dict()
                district.append(district_item)

        subdistrict: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.subdistrict, Unset):
            subdistrict = []
            for subdistrict_item_data in self.subdistrict:
                subdistrict_item = subdistrict_item_data.to_dict()
                subdistrict.append(subdistrict_item)

        street: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.street, Unset):
            street = []
            for street_item_data in self.street:
                street_item = street_item_data.to_dict()
                street.append(street_item)

        block: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.block, Unset):
            block = []
            for block_item_data in self.block:
                block_item = block_item_data.to_dict()
                block.append(block_item)

        subblock: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.subblock, Unset):
            subblock = []
            for subblock_item_data in self.subblock:
                subblock_item = subblock_item_data.to_dict()
                subblock.append(subblock_item)

        house_number: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.house_number, Unset):
            house_number = []
            for house_number_item_data in self.house_number:
                house_number_item = house_number_item_data.to_dict()
                house_number.append(house_number_item)

        postal_code: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.postal_code, Unset):
            postal_code = []
            for postal_code_item_data in self.postal_code:
                postal_code_item = postal_code_item_data.to_dict()
                postal_code.append(postal_code_item)

        building: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.building, Unset):
            building = []
            for building_item_data in self.building:
                building_item = building_item_data.to_dict()
                building.append(building_item)

        secondary_units: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.secondary_units, Unset):
            secondary_units = []
            for secondary_units_item_data in self.secondary_units:
                secondary_units_item = secondary_units_item_data.to_dict()
                secondary_units.append(secondary_units_item)

        additional_info: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.additional_info, Unset):
            additional_info = []
            for additional_info_item_data in self.additional_info:
                additional_info_item = additional_info_item_data.to_dict()
                additional_info.append(additional_info_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if place_name is not UNSET:
            field_dict["placeName"] = place_name
        if country is not UNSET:
            field_dict["country"] = country
        if state is not UNSET:
            field_dict["state"] = state
        if county is not UNSET:
            field_dict["county"] = county
        if city is not UNSET:
            field_dict["city"] = city
        if district is not UNSET:
            field_dict["district"] = district
        if subdistrict is not UNSET:
            field_dict["subdistrict"] = subdistrict
        if street is not UNSET:
            field_dict["street"] = street
        if block is not UNSET:
            field_dict["block"] = block
        if subblock is not UNSET:
            field_dict["subblock"] = subblock
        if house_number is not UNSET:
            field_dict["houseNumber"] = house_number
        if postal_code is not UNSET:
            field_dict["postalCode"] = postal_code
        if building is not UNSET:
            field_dict["building"] = building
        if secondary_units is not UNSET:
            field_dict["secondaryUnits"] = secondary_units
        if additional_info is not UNSET:
            field_dict["additionalInfo"] = additional_info

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.match_info import MatchInfo

        d = dict(src_dict)
        _place_name = d.pop("placeName", UNSET)
        place_name: list[MatchInfo] | Unset = UNSET
        if _place_name is not UNSET:
            place_name = []
            for place_name_item_data in _place_name:
                place_name_item = MatchInfo.from_dict(place_name_item_data)

                place_name.append(place_name_item)

        _country = d.pop("country", UNSET)
        country: list[MatchInfo] | Unset = UNSET
        if _country is not UNSET:
            country = []
            for country_item_data in _country:
                country_item = MatchInfo.from_dict(country_item_data)

                country.append(country_item)

        _state = d.pop("state", UNSET)
        state: list[MatchInfo] | Unset = UNSET
        if _state is not UNSET:
            state = []
            for state_item_data in _state:
                state_item = MatchInfo.from_dict(state_item_data)

                state.append(state_item)

        _county = d.pop("county", UNSET)
        county: list[MatchInfo] | Unset = UNSET
        if _county is not UNSET:
            county = []
            for county_item_data in _county:
                county_item = MatchInfo.from_dict(county_item_data)

                county.append(county_item)

        _city = d.pop("city", UNSET)
        city: list[MatchInfo] | Unset = UNSET
        if _city is not UNSET:
            city = []
            for city_item_data in _city:
                city_item = MatchInfo.from_dict(city_item_data)

                city.append(city_item)

        _district = d.pop("district", UNSET)
        district: list[MatchInfo] | Unset = UNSET
        if _district is not UNSET:
            district = []
            for district_item_data in _district:
                district_item = MatchInfo.from_dict(district_item_data)

                district.append(district_item)

        _subdistrict = d.pop("subdistrict", UNSET)
        subdistrict: list[MatchInfo] | Unset = UNSET
        if _subdistrict is not UNSET:
            subdistrict = []
            for subdistrict_item_data in _subdistrict:
                subdistrict_item = MatchInfo.from_dict(subdistrict_item_data)

                subdistrict.append(subdistrict_item)

        _street = d.pop("street", UNSET)
        street: list[MatchInfo] | Unset = UNSET
        if _street is not UNSET:
            street = []
            for street_item_data in _street:
                street_item = MatchInfo.from_dict(street_item_data)

                street.append(street_item)

        _block = d.pop("block", UNSET)
        block: list[MatchInfo] | Unset = UNSET
        if _block is not UNSET:
            block = []
            for block_item_data in _block:
                block_item = MatchInfo.from_dict(block_item_data)

                block.append(block_item)

        _subblock = d.pop("subblock", UNSET)
        subblock: list[MatchInfo] | Unset = UNSET
        if _subblock is not UNSET:
            subblock = []
            for subblock_item_data in _subblock:
                subblock_item = MatchInfo.from_dict(subblock_item_data)

                subblock.append(subblock_item)

        _house_number = d.pop("houseNumber", UNSET)
        house_number: list[MatchInfo] | Unset = UNSET
        if _house_number is not UNSET:
            house_number = []
            for house_number_item_data in _house_number:
                house_number_item = MatchInfo.from_dict(house_number_item_data)

                house_number.append(house_number_item)

        _postal_code = d.pop("postalCode", UNSET)
        postal_code: list[MatchInfo] | Unset = UNSET
        if _postal_code is not UNSET:
            postal_code = []
            for postal_code_item_data in _postal_code:
                postal_code_item = MatchInfo.from_dict(postal_code_item_data)

                postal_code.append(postal_code_item)

        _building = d.pop("building", UNSET)
        building: list[MatchInfo] | Unset = UNSET
        if _building is not UNSET:
            building = []
            for building_item_data in _building:
                building_item = MatchInfo.from_dict(building_item_data)

                building.append(building_item)

        _secondary_units = d.pop("secondaryUnits", UNSET)
        secondary_units: list[MatchInfo] | Unset = UNSET
        if _secondary_units is not UNSET:
            secondary_units = []
            for secondary_units_item_data in _secondary_units:
                secondary_units_item = MatchInfo.from_dict(secondary_units_item_data)

                secondary_units.append(secondary_units_item)

        _additional_info = d.pop("additionalInfo", UNSET)
        additional_info: list[MatchInfo] | Unset = UNSET
        if _additional_info is not UNSET:
            additional_info = []
            for additional_info_item_data in _additional_info:
                additional_info_item = MatchInfo.from_dict(additional_info_item_data)

                additional_info.append(additional_info_item)

        parsing = cls(
            place_name=place_name,
            country=country,
            state=state,
            county=county,
            city=city,
            district=district,
            subdistrict=subdistrict,
            street=street,
            block=block,
            subblock=subblock,
            house_number=house_number,
            postal_code=postal_code,
            building=building,
            secondary_units=secondary_units,
            additional_info=additional_info,
        )

        parsing.additional_properties = d
        return parsing

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="BaseEVEmpiricalModel")


@_attrs_define
class BaseEVEmpiricalModel:
    """Specifies parameters required for calculation of energy consumption for electric vehicles using Empirical model.

    Attributes:
        free_flow_speed_table (str): Function curve specifying consumption rate at a given speed.

            The format of the string is a comma-separated list of numbers, as follows:

            ```
            <SPEED_0>,<CONSUMPTION_0>,<SPEED_1>,<CONSUMPTION_1>,...,<SPEED_N>,<CONSUMPTION_N>
            ```

            where speed values are strictly increasing, non-negative integers in units of (km/h), and
            consumption values are non-negative floating point values.

            * Unit for EV:
            | Vehicle Type | Unit |
            |--------------|------|
            | Electric | Wh/m i.e., Watt-hours per meter|

            * Units for Fuel-based vehicles:
            | Vehicle Type | Unit |
            |--------------|------|
            | Diesel, Petrol & LPG | ml/m i.e., milliliters per meter|
            | CNG | gm/m i.e., grams per meter|

            The function is linearly interpolated between data points. For speeds less than `SPEED_0`
            the value of the function is `CONSUMPTION_0`, and for speeds greater than `SPEED_N` the
            value of the function is `CONSUMPTION_N`.
             Example:
            0,0.2394,14,0.2394,36,0.2586,52,0.196,68,0.2074,83,0.238,95,0.2597,105,0.2597,115,0.2964,125,0.3367,135,0.3508.
        traffic_speed_table (str | Unset): Function curve specifying consumption rate at a given speed.

            The format of the string is a comma-separated list of numbers, as follows:

            ```
            <SPEED_0>,<CONSUMPTION_0>,<SPEED_1>,<CONSUMPTION_1>,...,<SPEED_N>,<CONSUMPTION_N>
            ```

            where speed values are strictly increasing, non-negative integers in units of (km/h), and
            consumption values are non-negative floating point values.

            * Unit for EV:
            | Vehicle Type | Unit |
            |--------------|------|
            | Electric | Wh/m i.e., Watt-hours per meter|

            * Units for Fuel-based vehicles:
            | Vehicle Type | Unit |
            |--------------|------|
            | Diesel, Petrol & LPG | ml/m i.e., milliliters per meter|
            | CNG | gm/m i.e., grams per meter|

            The function is linearly interpolated between data points. For speeds less than `SPEED_0`
            the value of the function is `CONSUMPTION_0`, and for speeds greater than `SPEED_N` the
            value of the function is `CONSUMPTION_N`.
             Example:
            0,0.2394,14,0.2394,36,0.2586,52,0.196,68,0.2074,83,0.238,95,0.2597,105,0.2597,115,0.2964,125,0.3367,135,0.3508.
        ascent (float | Unset): Rate of energy consumed per meter rise in elevation (in Wh/m, i.e., Watt-hours per
            meter).
        descent (float | Unset): Rate of energy recovered per meter fall in elevation (in Wh/m, i.e., Watt-hours per
            meter).
        auxiliary_consumption (float | Unset): Rate of energy (in Wh/s) consumed by the vehicle's auxiliary systems (for
            example, air conditioning, lights).
            The value represents the number of Watt-hours consumed per second of travel.
    """

    free_flow_speed_table: str
    traffic_speed_table: str | Unset = UNSET
    ascent: float | Unset = UNSET
    descent: float | Unset = UNSET
    auxiliary_consumption: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        free_flow_speed_table = self.free_flow_speed_table

        traffic_speed_table = self.traffic_speed_table

        ascent = self.ascent

        descent = self.descent

        auxiliary_consumption = self.auxiliary_consumption

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "freeFlowSpeedTable": free_flow_speed_table,
        })
        if traffic_speed_table is not UNSET:
            field_dict["trafficSpeedTable"] = traffic_speed_table
        if ascent is not UNSET:
            field_dict["ascent"] = ascent
        if descent is not UNSET:
            field_dict["descent"] = descent
        if auxiliary_consumption is not UNSET:
            field_dict["auxiliaryConsumption"] = auxiliary_consumption

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        free_flow_speed_table = d.pop("freeFlowSpeedTable")

        traffic_speed_table = d.pop("trafficSpeedTable", UNSET)

        ascent = d.pop("ascent", UNSET)

        descent = d.pop("descent", UNSET)

        auxiliary_consumption = d.pop("auxiliaryConsumption", UNSET)

        base_ev_empirical_model = cls(
            free_flow_speed_table=free_flow_speed_table,
            traffic_speed_table=traffic_speed_table,
            ascent=ascent,
            descent=descent,
            auxiliary_consumption=auxiliary_consumption,
        )

        base_ev_empirical_model.additional_properties = d
        return base_ev_empirical_model

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.autosuggest_query_result_item_result_type import (
    AutosuggestQueryResultItemResultType,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.title_highlighting import TitleHighlighting


T = TypeVar("T", bound="AutosuggestQueryResultItem")


@_attrs_define
class AutosuggestQueryResultItem:
    """
    Attributes:
        title (str): The localized display name of this result item.
        id (str): The unique identifier for the result item. The identifier of an `AutosuggestQueryResultItem` cannot be
            used for a Lookup by ID search.
        result_type (AutosuggestQueryResultItemResultType | Unset):
            Type of the result item.

            Note: `addressBlock` result item is either a block or subblock.

            `resultType` values can get added to the list without further notice.
        no_result_on_follow_up (bool | Unset): `noResultOnFollowUp` is added with a value `true` when the `chainQuery`
            response item is providing a
            follow-up `href` likely to not lead to any result. If `noResultOnFollowUp` is not added, it is likely the
            follow-up `href` leads to places near the search center.
            Customers can use the `noResultOnFollowUp`, for example to not expose the `chainQuery` response item in their
            application, or
            signal where there is "no nearby result for the chain query".
        href (str | Unset): URL of the follow-up query
        highlights (TitleHighlighting | Unset):
    """

    title: str
    id: str
    result_type: AutosuggestQueryResultItemResultType | Unset = UNSET
    no_result_on_follow_up: bool | Unset = UNSET
    href: str | Unset = UNSET
    highlights: TitleHighlighting | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        title = self.title

        id = self.id

        result_type: str | Unset = UNSET
        if not isinstance(self.result_type, Unset):
            result_type = self.result_type.value

        no_result_on_follow_up = self.no_result_on_follow_up

        href = self.href

        highlights: dict[str, Any] | Unset = UNSET
        if not isinstance(self.highlights, Unset):
            highlights = self.highlights.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "title": title,
            "id": id,
        })
        if result_type is not UNSET:
            field_dict["resultType"] = result_type
        if no_result_on_follow_up is not UNSET:
            field_dict["noResultOnFollowUp"] = no_result_on_follow_up
        if href is not UNSET:
            field_dict["href"] = href
        if highlights is not UNSET:
            field_dict["highlights"] = highlights

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.title_highlighting import TitleHighlighting

        d = dict(src_dict)
        title = d.pop("title")

        id = d.pop("id")

        _result_type = d.pop("resultType", UNSET)
        result_type: AutosuggestQueryResultItemResultType | Unset
        if isinstance(_result_type, Unset):
            result_type = UNSET
        else:
            result_type = AutosuggestQueryResultItemResultType(_result_type)

        no_result_on_follow_up = d.pop("noResultOnFollowUp", UNSET)

        href = d.pop("href", UNSET)

        _highlights = d.pop("highlights", UNSET)
        highlights: TitleHighlighting | Unset
        if isinstance(_highlights, Unset):
            highlights = UNSET
        else:
            highlights = TitleHighlighting.from_dict(_highlights)

        autosuggest_query_result_item = cls(
            title=title,
            id=id,
            result_type=result_type,
            no_result_on_follow_up=no_result_on_follow_up,
            href=href,
            highlights=highlights,
        )

        autosuggest_query_result_item.additional_properties = d
        return autosuggest_query_result_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.transit_section_type import TransitSectionType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.agency import Agency
    from ..models.arrive_action import ArriveAction
    from ..models.attribution import Attribution
    from ..models.base_summary import BaseSummary
    from ..models.board_action import BoardAction
    from ..models.continue_action import ContinueAction
    from ..models.deboard_action import DeboardAction
    from ..models.depart_action import DepartAction
    from ..models.fare import Fare
    from ..models.passthrough import Passthrough
    from ..models.ref_replacements import RefReplacements
    from ..models.transit_departure import TransitDeparture
    from ..models.transit_incident import TransitIncident
    from ..models.transit_notice import TransitNotice
    from ..models.transit_span import TransitSpan
    from ..models.transit_stop import TransitStop
    from ..models.transit_transport import TransitTransport
    from ..models.web_link import WebLink
    from ..models.web_link_with_device_type import WebLinkWithDeviceType


T = TypeVar("T", bound="TransitSection")


@_attrs_define
class TransitSection:
    """A section of the route. It contains departure, arrival, and route information.

    Attributes:
        id (str): Unique identifier of the section
        type_ (TransitSectionType): Section type used by the client to identify what extension to the BaseSection are
            available.
        departure (TransitDeparture): Transit departure
        arrival (TransitDeparture): Transit departure
        pre_actions (list[BoardAction] | Unset): Actions that must be done prior to `departure`.
        actions (list[ArriveAction | ContinueAction | DepartAction] | Unset): Actions that must be done during the
            travel portion of the section.

            Action offsets represent the coordinate index in the polyline.

            *NOTE:* currentRoad and nextRoad are not populated for actions.
        language (str | Unset): Language of the localized strings in the section, if any, in BCP47 format.
        post_actions (list[DeboardAction] | Unset): Actions that must be done after `arrival`.
        turn_by_turn_actions (list[DepartAction] | Unset): Actions for turn by turn guidance during the travel portion
            of the section, i.e., between `departure` and `arrival`.
        passthrough (list[Passthrough] | Unset): List of via waypoints this section is passing through.

            Each via waypoint of the request that is a `passThrough=true` waypoint, appears as a
            `Passthrough` in the response. It appears in the section that starts with the closest
            non-passthrough via specified before it or origin.

            The passthrough vias appear in this list in the order they are traversed. They are
            traversed in the order they are specified in the request.
        summary (BaseSummary | Unset): Total value of key attributes for a route section.
        travel_summary (BaseSummary | Unset): Total value of key attributes for a route section.
        polyline (str | Unset): Line string in [Flexible Polyline](https://github.com/heremaps/flexible-polyline)
            format. Coordinates are in the WGS84 coordinate system, including `Elevation` (if present). Example:
            A05xgKuy2xCx9B7vUl0OhnR54EqSzpEl-HxjD3pBiGnyGi2CvwFsgD3nD4vB6e.
        notices (list[TransitNotice] | Unset): Contains a list of issues related to this section of the route.
        booking_links (list[WebLinkWithDeviceType] | Unset): Links to external ticket booking services
        transport (TransitTransport | Unset): Transit transport information.
        intermediate_stops (list[TransitStop] | Unset): Intermediate stops between departure and destination of the
            transit line. It can be missing if this information is not available or not requested.
        agency (Agency | Unset): Contains information about a particular agency.
        attributions (list[Attribution] | Unset): List of required attributions to display.
        fares (list[Fare] | Unset): "List of tickets to pay for this section of the route"

            **NOTE**: Fares information support will be available soon.
        booking (WebLink | Unset): The URL address to an external resource. Example: {'id': '88-7568-21.07.2023',
            'text': 'Information for public transit provided by ThePublicTransit GmbH'}.
        spans (list[TransitSpan] | Unset): Span attached to a `Section` describing transit content.
        ref_replacements (RefReplacements | Unset): Dictionary of placeholders to replacement strings for the compact
            representation of map entity references.
        incidents (list[TransitIncident] | Unset): A list of all incidents that apply to the section.
    """

    id: str
    type_: TransitSectionType
    departure: TransitDeparture
    arrival: TransitDeparture
    pre_actions: list[BoardAction] | Unset = UNSET
    actions: list[ArriveAction | ContinueAction | DepartAction] | Unset = UNSET
    language: str | Unset = UNSET
    post_actions: list[DeboardAction] | Unset = UNSET
    turn_by_turn_actions: list[DepartAction] | Unset = UNSET
    passthrough: list[Passthrough] | Unset = UNSET
    summary: BaseSummary | Unset = UNSET
    travel_summary: BaseSummary | Unset = UNSET
    polyline: str | Unset = UNSET
    notices: list[TransitNotice] | Unset = UNSET
    booking_links: list[WebLinkWithDeviceType] | Unset = UNSET
    transport: TransitTransport | Unset = UNSET
    intermediate_stops: list[TransitStop] | Unset = UNSET
    agency: Agency | Unset = UNSET
    attributions: list[Attribution] | Unset = UNSET
    fares: list[Fare] | Unset = UNSET
    booking: WebLink | Unset = UNSET
    spans: list[TransitSpan] | Unset = UNSET
    ref_replacements: RefReplacements | Unset = UNSET
    incidents: list[TransitIncident] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.arrive_action import ArriveAction
        from ..models.depart_action import DepartAction

        id = self.id

        type_ = self.type_.value

        departure = self.departure.to_dict()

        arrival = self.arrival.to_dict()

        pre_actions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.pre_actions, Unset):
            pre_actions = []
            for pre_actions_item_data in self.pre_actions:
                pre_actions_item = pre_actions_item_data.to_dict()
                pre_actions.append(pre_actions_item)

        actions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.actions, Unset):
            actions = []
            for actions_item_data in self.actions:
                actions_item: dict[str, Any]
                if isinstance(actions_item_data, DepartAction):
                    actions_item = actions_item_data.to_dict()
                elif isinstance(actions_item_data, ArriveAction):
                    actions_item = actions_item_data.to_dict()
                else:
                    actions_item = actions_item_data.to_dict()

                actions.append(actions_item)

        language = self.language

        post_actions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.post_actions, Unset):
            post_actions = []
            for post_actions_item_data in self.post_actions:
                post_actions_item = post_actions_item_data.to_dict()
                post_actions.append(post_actions_item)

        turn_by_turn_actions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.turn_by_turn_actions, Unset):
            turn_by_turn_actions = []
            for turn_by_turn_actions_item_data in self.turn_by_turn_actions:
                turn_by_turn_actions_item = turn_by_turn_actions_item_data.to_dict()
                turn_by_turn_actions.append(turn_by_turn_actions_item)

        passthrough: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.passthrough, Unset):
            passthrough = []
            for passthrough_item_data in self.passthrough:
                passthrough_item = passthrough_item_data.to_dict()
                passthrough.append(passthrough_item)

        summary: dict[str, Any] | Unset = UNSET
        if not isinstance(self.summary, Unset):
            summary = self.summary.to_dict()

        travel_summary: dict[str, Any] | Unset = UNSET
        if not isinstance(self.travel_summary, Unset):
            travel_summary = self.travel_summary.to_dict()

        polyline = self.polyline

        notices: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.notices, Unset):
            notices = []
            for notices_item_data in self.notices:
                notices_item = notices_item_data.to_dict()
                notices.append(notices_item)

        booking_links: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.booking_links, Unset):
            booking_links = []
            for booking_links_item_data in self.booking_links:
                booking_links_item = booking_links_item_data.to_dict()
                booking_links.append(booking_links_item)

        transport: dict[str, Any] | Unset = UNSET
        if not isinstance(self.transport, Unset):
            transport = self.transport.to_dict()

        intermediate_stops: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.intermediate_stops, Unset):
            intermediate_stops = []
            for intermediate_stops_item_data in self.intermediate_stops:
                intermediate_stops_item = intermediate_stops_item_data.to_dict()
                intermediate_stops.append(intermediate_stops_item)

        agency: dict[str, Any] | Unset = UNSET
        if not isinstance(self.agency, Unset):
            agency = self.agency.to_dict()

        attributions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.attributions, Unset):
            attributions = []
            for attributions_item_data in self.attributions:
                attributions_item = attributions_item_data.to_dict()
                attributions.append(attributions_item)

        fares: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.fares, Unset):
            fares = []
            for fares_item_data in self.fares:
                fares_item = fares_item_data.to_dict()
                fares.append(fares_item)

        booking: dict[str, Any] | Unset = UNSET
        if not isinstance(self.booking, Unset):
            booking = self.booking.to_dict()

        spans: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.spans, Unset):
            spans = []
            for spans_item_data in self.spans:
                spans_item = spans_item_data.to_dict()
                spans.append(spans_item)

        ref_replacements: dict[str, Any] | Unset = UNSET
        if not isinstance(self.ref_replacements, Unset):
            ref_replacements = self.ref_replacements.to_dict()

        incidents: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.incidents, Unset):
            incidents = []
            for incidents_item_data in self.incidents:
                incidents_item = incidents_item_data.to_dict()
                incidents.append(incidents_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "type": type_,
            "departure": departure,
            "arrival": arrival,
        })
        if pre_actions is not UNSET:
            field_dict["preActions"] = pre_actions
        if actions is not UNSET:
            field_dict["actions"] = actions
        if language is not UNSET:
            field_dict["language"] = language
        if post_actions is not UNSET:
            field_dict["postActions"] = post_actions
        if turn_by_turn_actions is not UNSET:
            field_dict["turnByTurnActions"] = turn_by_turn_actions
        if passthrough is not UNSET:
            field_dict["passthrough"] = passthrough
        if summary is not UNSET:
            field_dict["summary"] = summary
        if travel_summary is not UNSET:
            field_dict["travelSummary"] = travel_summary
        if polyline is not UNSET:
            field_dict["polyline"] = polyline
        if notices is not UNSET:
            field_dict["notices"] = notices
        if booking_links is not UNSET:
            field_dict["bookingLinks"] = booking_links
        if transport is not UNSET:
            field_dict["transport"] = transport
        if intermediate_stops is not UNSET:
            field_dict["intermediateStops"] = intermediate_stops
        if agency is not UNSET:
            field_dict["agency"] = agency
        if attributions is not UNSET:
            field_dict["attributions"] = attributions
        if fares is not UNSET:
            field_dict["fares"] = fares
        if booking is not UNSET:
            field_dict["booking"] = booking
        if spans is not UNSET:
            field_dict["spans"] = spans
        if ref_replacements is not UNSET:
            field_dict["refReplacements"] = ref_replacements
        if incidents is not UNSET:
            field_dict["incidents"] = incidents

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.agency import Agency
        from ..models.arrive_action import ArriveAction
        from ..models.attribution import Attribution
        from ..models.base_summary import BaseSummary
        from ..models.board_action import BoardAction
        from ..models.continue_action import ContinueAction
        from ..models.deboard_action import DeboardAction
        from ..models.depart_action import DepartAction
        from ..models.fare import Fare
        from ..models.passthrough import Passthrough
        from ..models.ref_replacements import RefReplacements
        from ..models.transit_departure import TransitDeparture
        from ..models.transit_incident import TransitIncident
        from ..models.transit_notice import TransitNotice
        from ..models.transit_span import TransitSpan
        from ..models.transit_stop import TransitStop
        from ..models.transit_transport import TransitTransport
        from ..models.web_link import WebLink
        from ..models.web_link_with_device_type import WebLinkWithDeviceType

        d = dict(src_dict)
        id = d.pop("id")

        type_ = TransitSectionType(d.pop("type"))

        departure = TransitDeparture.from_dict(d.pop("departure"))

        arrival = TransitDeparture.from_dict(d.pop("arrival"))

        _pre_actions = d.pop("preActions", UNSET)
        pre_actions: list[BoardAction] | Unset = UNSET
        if _pre_actions is not UNSET:
            pre_actions = []
            for pre_actions_item_data in _pre_actions:
                pre_actions_item = BoardAction.from_dict(pre_actions_item_data)

                pre_actions.append(pre_actions_item)

        _actions = d.pop("actions", UNSET)
        actions: list[ArriveAction | ContinueAction | DepartAction] | Unset = UNSET
        if _actions is not UNSET:
            actions = []
            for actions_item_data in _actions:

                def _parse_actions_item(
                    data: object,
                ) -> ArriveAction | ContinueAction | DepartAction:
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_transit_action_type_0 = (
                            DepartAction.from_dict(data)
                        )

                        return componentsschemas_transit_action_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_transit_action_type_1 = (
                            ArriveAction.from_dict(data)
                        )

                        return componentsschemas_transit_action_type_1
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_transit_action_type_2 = ContinueAction.from_dict(
                        data
                    )

                    return componentsschemas_transit_action_type_2

                actions_item = _parse_actions_item(actions_item_data)

                actions.append(actions_item)

        language = d.pop("language", UNSET)

        _post_actions = d.pop("postActions", UNSET)
        post_actions: list[DeboardAction] | Unset = UNSET
        if _post_actions is not UNSET:
            post_actions = []
            for post_actions_item_data in _post_actions:
                post_actions_item = DeboardAction.from_dict(post_actions_item_data)

                post_actions.append(post_actions_item)

        _turn_by_turn_actions = d.pop("turnByTurnActions", UNSET)
        turn_by_turn_actions: list[DepartAction] | Unset = UNSET
        if _turn_by_turn_actions is not UNSET:
            turn_by_turn_actions = []
            for turn_by_turn_actions_item_data in _turn_by_turn_actions:
                turn_by_turn_actions_item = DepartAction.from_dict(
                    turn_by_turn_actions_item_data
                )

                turn_by_turn_actions.append(turn_by_turn_actions_item)

        _passthrough = d.pop("passthrough", UNSET)
        passthrough: list[Passthrough] | Unset = UNSET
        if _passthrough is not UNSET:
            passthrough = []
            for passthrough_item_data in _passthrough:
                passthrough_item = Passthrough.from_dict(passthrough_item_data)

                passthrough.append(passthrough_item)

        _summary = d.pop("summary", UNSET)
        summary: BaseSummary | Unset
        if isinstance(_summary, Unset):
            summary = UNSET
        else:
            summary = BaseSummary.from_dict(_summary)

        _travel_summary = d.pop("travelSummary", UNSET)
        travel_summary: BaseSummary | Unset
        if isinstance(_travel_summary, Unset):
            travel_summary = UNSET
        else:
            travel_summary = BaseSummary.from_dict(_travel_summary)

        polyline = d.pop("polyline", UNSET)

        _notices = d.pop("notices", UNSET)
        notices: list[TransitNotice] | Unset = UNSET
        if _notices is not UNSET:
            notices = []
            for notices_item_data in _notices:
                notices_item = TransitNotice.from_dict(notices_item_data)

                notices.append(notices_item)

        _booking_links = d.pop("bookingLinks", UNSET)
        booking_links: list[WebLinkWithDeviceType] | Unset = UNSET
        if _booking_links is not UNSET:
            booking_links = []
            for booking_links_item_data in _booking_links:
                booking_links_item = WebLinkWithDeviceType.from_dict(
                    booking_links_item_data
                )

                booking_links.append(booking_links_item)

        _transport = d.pop("transport", UNSET)
        transport: TransitTransport | Unset
        if isinstance(_transport, Unset):
            transport = UNSET
        else:
            transport = TransitTransport.from_dict(_transport)

        _intermediate_stops = d.pop("intermediateStops", UNSET)
        intermediate_stops: list[TransitStop] | Unset = UNSET
        if _intermediate_stops is not UNSET:
            intermediate_stops = []
            for intermediate_stops_item_data in _intermediate_stops:
                intermediate_stops_item = TransitStop.from_dict(
                    intermediate_stops_item_data
                )

                intermediate_stops.append(intermediate_stops_item)

        _agency = d.pop("agency", UNSET)
        agency: Agency | Unset
        if isinstance(_agency, Unset):
            agency = UNSET
        else:
            agency = Agency.from_dict(_agency)

        _attributions = d.pop("attributions", UNSET)
        attributions: list[Attribution] | Unset = UNSET
        if _attributions is not UNSET:
            attributions = []
            for attributions_item_data in _attributions:
                attributions_item = Attribution.from_dict(attributions_item_data)

                attributions.append(attributions_item)

        _fares = d.pop("fares", UNSET)
        fares: list[Fare] | Unset = UNSET
        if _fares is not UNSET:
            fares = []
            for fares_item_data in _fares:
                fares_item = Fare.from_dict(fares_item_data)

                fares.append(fares_item)

        _booking = d.pop("booking", UNSET)
        booking: WebLink | Unset
        if isinstance(_booking, Unset):
            booking = UNSET
        else:
            booking = WebLink.from_dict(_booking)

        _spans = d.pop("spans", UNSET)
        spans: list[TransitSpan] | Unset = UNSET
        if _spans is not UNSET:
            spans = []
            for spans_item_data in _spans:
                spans_item = TransitSpan.from_dict(spans_item_data)

                spans.append(spans_item)

        _ref_replacements = d.pop("refReplacements", UNSET)
        ref_replacements: RefReplacements | Unset
        if isinstance(_ref_replacements, Unset):
            ref_replacements = UNSET
        else:
            ref_replacements = RefReplacements.from_dict(_ref_replacements)

        _incidents = d.pop("incidents", UNSET)
        incidents: list[TransitIncident] | Unset = UNSET
        if _incidents is not UNSET:
            incidents = []
            for incidents_item_data in _incidents:
                incidents_item = TransitIncident.from_dict(incidents_item_data)

                incidents.append(incidents_item)

        transit_section = cls(
            id=id,
            type_=type_,
            departure=departure,
            arrival=arrival,
            pre_actions=pre_actions,
            actions=actions,
            language=language,
            post_actions=post_actions,
            turn_by_turn_actions=turn_by_turn_actions,
            passthrough=passthrough,
            summary=summary,
            travel_summary=travel_summary,
            polyline=polyline,
            notices=notices,
            booking_links=booking_links,
            transport=transport,
            intermediate_stops=intermediate_stops,
            agency=agency,
            attributions=attributions,
            fares=fares,
            booking=booking,
            spans=spans,
            ref_replacements=ref_replacements,
            incidents=incidents,
        )

        transit_section.additional_properties = d
        return transit_section

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.access_point_place import AccessPointPlace
    from ..models.charging_station_place import ChargingStationPlace
    from ..models.docking_station_place import DockingStationPlace
    from ..models.parking_lot_place import ParkingLotPlace
    from ..models.place import Place
    from ..models.station_place import StationPlace


T = TypeVar("T", bound="PedestrianDeparture")


@_attrs_define
class PedestrianDeparture:
    """Departure of pedestrian

    Attributes:
        place (AccessPointPlace | ChargingStationPlace | DockingStationPlace | ParkingLotPlace | Place | StationPlace):
            Place used in pedestrian routing
        time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either `date-time` or `date-only` 'T'
            `partial-time` (ie no time-offset).
    """

    place: (
        AccessPointPlace
        | ChargingStationPlace
        | DockingStationPlace
        | ParkingLotPlace
        | Place
        | StationPlace
    )
    time: datetime.datetime | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.access_point_place import AccessPointPlace
        from ..models.charging_station_place import ChargingStationPlace
        from ..models.parking_lot_place import ParkingLotPlace
        from ..models.place import Place
        from ..models.station_place import StationPlace

        place: dict[str, Any]
        if isinstance(self.place, Place):
            place = self.place.to_dict()
        elif isinstance(self.place, StationPlace):
            place = self.place.to_dict()
        elif isinstance(self.place, AccessPointPlace):
            place = self.place.to_dict()
        elif isinstance(self.place, ParkingLotPlace):
            place = self.place.to_dict()
        elif isinstance(self.place, ChargingStationPlace):
            place = self.place.to_dict()
        else:
            place = self.place.to_dict()

        time: str | Unset = UNSET
        if not isinstance(self.time, Unset):
            time = self.time.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "place": place,
        })
        if time is not UNSET:
            field_dict["time"] = time

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.access_point_place import AccessPointPlace
        from ..models.charging_station_place import ChargingStationPlace
        from ..models.docking_station_place import DockingStationPlace
        from ..models.parking_lot_place import ParkingLotPlace
        from ..models.place import Place
        from ..models.station_place import StationPlace

        d = dict(src_dict)

        def _parse_place(
            data: object,
        ) -> (
            AccessPointPlace
            | ChargingStationPlace
            | DockingStationPlace
            | ParkingLotPlace
            | Place
            | StationPlace
        ):
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_pedestrian_place_type_0 = Place.from_dict(data)

                return componentsschemas_pedestrian_place_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_pedestrian_place_type_1 = StationPlace.from_dict(data)

                return componentsschemas_pedestrian_place_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_pedestrian_place_type_2 = AccessPointPlace.from_dict(
                    data
                )

                return componentsschemas_pedestrian_place_type_2
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_pedestrian_place_type_3 = ParkingLotPlace.from_dict(
                    data
                )

                return componentsschemas_pedestrian_place_type_3
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_pedestrian_place_type_4 = (
                    ChargingStationPlace.from_dict(data)
                )

                return componentsschemas_pedestrian_place_type_4
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_pedestrian_place_type_5 = DockingStationPlace.from_dict(
                data
            )

            return componentsschemas_pedestrian_place_type_5

        place = _parse_place(d.pop("place"))

        _time = d.pop("time", UNSET)
        time: datetime.datetime | Unset
        if isinstance(_time, Unset):
            time = UNSET
        else:
            time = isoparse(_time)

        pedestrian_departure = cls(
            place=place,
            time=time,
        )

        pedestrian_departure.additional_properties = d
        return pedestrian_departure

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="FuelPrice")


@_attrs_define
class FuelPrice:
    """
    Attributes:
        amount (float | Unset): **BETA, RESTRICTED**

            Real-time price value for a volume of 1 unit
        currency (str | Unset): **BETA, RESTRICTED**

            Currency in which the amount is expressed
        unit (str | Unset): **BETA, RESTRICTED**

            Unit of measurement for the fuel quantity:
            - `l` litres
            - `gal` gallons
            - `bbl` barrels
            - `kg` kilogram
            - `m3` cubic meters
    """

    amount: float | Unset = UNSET
    currency: str | Unset = UNSET
    unit: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        amount = self.amount

        currency = self.currency

        unit = self.unit

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if amount is not UNSET:
            field_dict["amount"] = amount
        if currency is not UNSET:
            field_dict["currency"] = currency
        if unit is not UNSET:
            field_dict["unit"] = unit

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        amount = d.pop("amount", UNSET)

        currency = d.pop("currency", UNSET)

        unit = d.pop("unit", UNSET)

        fuel_price = cls(
            amount=amount,
            currency=currency,
            unit=unit,
        )

        fuel_price.additional_properties = d
        return fuel_price

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.range_price import RangePrice
    from ..models.single_price import SinglePrice
    from ..models.toll_country_summary import TollCountrySummary
    from ..models.toll_system_summary import TollSystemSummary


T = TypeVar("T", bound="TollSummary")


@_attrs_define
class TollSummary:
    """The summary of the tolls grouped by criteria (total, per system, per country).

    Attributes:
        total (RangePrice | SinglePrice | Unset): Price of a fare
        total_by_system (list[TollSystemSummary] | Unset): Categorizes toll fees based on the different toll systems
            applicable along the route or section. Each toll system has its own set of charges and payment rules. The
            summary provides a breakdown of toll costs for each individual toll system encountered.
        total_by_country (list[TollCountrySummary] | Unset): Groups toll fees based on the countries traversed during
            the route or section. It provides a breakdown of toll charges specific to each country.
    """

    total: RangePrice | SinglePrice | Unset = UNSET
    total_by_system: list[TollSystemSummary] | Unset = UNSET
    total_by_country: list[TollCountrySummary] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.single_price import SinglePrice

        total: dict[str, Any] | Unset
        if isinstance(self.total, Unset):
            total = UNSET
        elif isinstance(self.total, SinglePrice):
            total = self.total.to_dict()
        else:
            total = self.total.to_dict()

        total_by_system: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.total_by_system, Unset):
            total_by_system = []
            for total_by_system_item_data in self.total_by_system:
                total_by_system_item = total_by_system_item_data.to_dict()
                total_by_system.append(total_by_system_item)

        total_by_country: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.total_by_country, Unset):
            total_by_country = []
            for total_by_country_item_data in self.total_by_country:
                total_by_country_item = total_by_country_item_data.to_dict()
                total_by_country.append(total_by_country_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if total is not UNSET:
            field_dict["total"] = total
        if total_by_system is not UNSET:
            field_dict["totalBySystem"] = total_by_system
        if total_by_country is not UNSET:
            field_dict["totalByCountry"] = total_by_country

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.range_price import RangePrice
        from ..models.single_price import SinglePrice
        from ..models.toll_country_summary import TollCountrySummary
        from ..models.toll_system_summary import TollSystemSummary

        d = dict(src_dict)

        def _parse_total(data: object) -> RangePrice | SinglePrice | Unset:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_fare_price_type_0 = SinglePrice.from_dict(data)

                return componentsschemas_fare_price_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_fare_price_type_1 = RangePrice.from_dict(data)

            return componentsschemas_fare_price_type_1

        total = _parse_total(d.pop("total", UNSET))

        _total_by_system = d.pop("totalBySystem", UNSET)
        total_by_system: list[TollSystemSummary] | Unset = UNSET
        if _total_by_system is not UNSET:
            total_by_system = []
            for total_by_system_item_data in _total_by_system:
                total_by_system_item = TollSystemSummary.from_dict(
                    total_by_system_item_data
                )

                total_by_system.append(total_by_system_item)

        _total_by_country = d.pop("totalByCountry", UNSET)
        total_by_country: list[TollCountrySummary] | Unset = UNSET
        if _total_by_country is not UNSET:
            total_by_country = []
            for total_by_country_item_data in _total_by_country:
                total_by_country_item = TollCountrySummary.from_dict(
                    total_by_country_item_data
                )

                total_by_country.append(total_by_country_item)

        toll_summary = cls(
            total=total,
            total_by_system=total_by_system,
            total_by_country=total_by_country,
        )

        toll_summary.additional_properties = d
        return toll_summary

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.related_address_house_number_type import RelatedAddressHouseNumberType
from ..models.related_address_relationship import RelatedAddressRelationship
from ..models.related_address_result_type import RelatedAddressResultType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.basic_access_point import BasicAccessPoint
    from ..models.display_response_coordinate import DisplayResponseCoordinate
    from ..models.map_reference_section import MapReferenceSection
    from ..models.map_view import MapView
    from ..models.related_result_address import RelatedResultAddress


T = TypeVar("T", bound="RelatedAddress")


@_attrs_define
class RelatedAddress:
    """
    Attributes:
        relationship (RelatedAddressRelationship):

            Description of supported values:

            - **RESTRICTED** `MPA`: Micro Point Addresses related to a result items with
              `resultType` = `houseNumber` and `houseNumberType` = `PA`;
              for example, buildings, floors (levels) or suites (units)
              associated to this address. This type of relationship is
              only returned by the Lookup endpoint and the Geocode endpoint
              when the `showRelated=MPA` parameter is provided.
            - `intersection`: The Intersections nearest to the address. This type of relationship is
              only returned by Reverse Geocode, Geocode or Multi-Reverse Geocode
              endpoints when `showRelated=intersections` parameter is provided.
            - `nearbyAddress`: Nearby address on the same street of main result. This type of
              relationship is only returned by Reverse Geocode endpoint or
              Multi-Reverse Geocode endpoint when `showRelated=nearbyAddress`
              parameter is provided.
            - **RESTRICTED** `parentPA`: The Point Address to which the Micro Point Address
              belongs; for example the house number to which this
              suite is associated. This type of relationship is only
              returned by the Lookup, the Geocode and the (Multi) Reverse Geocode endpoints when
              the `showRelated=parentPA` parameter is provided.
        id (str): The ID of the related entity.
        title (str | Unset): The localized display name of this related entity.
        result_type (RelatedAddressResultType | Unset): The resultType of the related entity.
        house_number_type (RelatedAddressHouseNumberType | Unset): The houseNumberType of the related entity.

            Description of supported values:

            - **RESTRICTED** `MPA`: A Micro Point Address represents a secondary address for a Point
              Address; for example, building, floor (level) and suite (unit).
              Micro Point Addresses can be used to enhance Point Address with
              greater address detail and higher coordinate accuracy. This result
              type is only returned by Lookup, Geocode or (Multi) Reverse Geocode endpoints when
              `with=MPA` parameter is provided.
            - `PA`: A Point Address represents an individual address as a point object.
              Point Addresses are coming from trusted sources. There is a
              high certainty that the address exists at that position. A
              Point Address result contains two types of coordinates. One is the
              access point (or navigation coordinates), which is the point to
              start or end a drive. The other point is the position or display
              point. This point varies per source and country. The point can be
              the rooftop point, a point close to the building entry, or a point
              close to the building, driveway or parking lot that belongs to the
              building.
            - `interpolated`: An interpolated address. These are approximate positions as a
              result of a linear interpolation based on address ranges. Address
              ranges, especially in the USA, are typical per block. For
              interpolated addresses, we cannot say with confidence that the
              address exists in reality. But the interpolation provides a good
              location approximation that brings people in most use cases close
              to the target location. The access point of an interpolated address
              result is calculated based on the address range and the road
              geometry. The position (display) point is pre-configured offset
              from the street geometry. Compared to Point Addresses, interpolated
              addresses are less accurate.
        address (RelatedResultAddress | Unset):
        position (DisplayResponseCoordinate | Unset):
        access (list[BasicAccessPoint] | Unset): Coordinates of the response item on a HERE map navigable link (for
            example, for driving or walking).
        distance (int | Unset): The distance "as the crow flies" from the search center to this related address'
            position in meters.
            It is only returned by Reverse Geocode endpoint or Multi-Reverse Geocode endpoint.
        route_distance (int | Unset): The distance from routing position of the nearby address to the street result, the
            distance is
            calculated based on the underlying road geometry. It is only returned by Reverse Geocode endpoint or
            Multi-Reverse Geocode endpoint.
        bearing (int | Unset): Bearing (measured clockwise from true north) from the projected coordinate on the street
            to the
            display position of the related location. True clockwise degree, 0 is north. It is only returned by
            Reverse Geocode endpoint or Multi-Reverse Geocode endpoint.
        map_view (MapView | Unset):
        map_references (MapReferenceSection | Unset):
    """

    relationship: RelatedAddressRelationship
    id: str
    title: str | Unset = UNSET
    result_type: RelatedAddressResultType | Unset = UNSET
    house_number_type: RelatedAddressHouseNumberType | Unset = UNSET
    address: RelatedResultAddress | Unset = UNSET
    position: DisplayResponseCoordinate | Unset = UNSET
    access: list[BasicAccessPoint] | Unset = UNSET
    distance: int | Unset = UNSET
    route_distance: int | Unset = UNSET
    bearing: int | Unset = UNSET
    map_view: MapView | Unset = UNSET
    map_references: MapReferenceSection | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        relationship = self.relationship.value

        id = self.id

        title = self.title

        result_type: str | Unset = UNSET
        if not isinstance(self.result_type, Unset):
            result_type = self.result_type.value

        house_number_type: str | Unset = UNSET
        if not isinstance(self.house_number_type, Unset):
            house_number_type = self.house_number_type.value

        address: dict[str, Any] | Unset = UNSET
        if not isinstance(self.address, Unset):
            address = self.address.to_dict()

        position: dict[str, Any] | Unset = UNSET
        if not isinstance(self.position, Unset):
            position = self.position.to_dict()

        access: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.access, Unset):
            access = []
            for access_item_data in self.access:
                access_item = access_item_data.to_dict()
                access.append(access_item)

        distance = self.distance

        route_distance = self.route_distance

        bearing = self.bearing

        map_view: dict[str, Any] | Unset = UNSET
        if not isinstance(self.map_view, Unset):
            map_view = self.map_view.to_dict()

        map_references: dict[str, Any] | Unset = UNSET
        if not isinstance(self.map_references, Unset):
            map_references = self.map_references.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "relationship": relationship,
            "id": id,
        })
        if title is not UNSET:
            field_dict["title"] = title
        if result_type is not UNSET:
            field_dict["resultType"] = result_type
        if house_number_type is not UNSET:
            field_dict["houseNumberType"] = house_number_type
        if address is not UNSET:
            field_dict["address"] = address
        if position is not UNSET:
            field_dict["position"] = position
        if access is not UNSET:
            field_dict["access"] = access
        if distance is not UNSET:
            field_dict["distance"] = distance
        if route_distance is not UNSET:
            field_dict["routeDistance"] = route_distance
        if bearing is not UNSET:
            field_dict["bearing"] = bearing
        if map_view is not UNSET:
            field_dict["mapView"] = map_view
        if map_references is not UNSET:
            field_dict["mapReferences"] = map_references

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.basic_access_point import BasicAccessPoint
        from ..models.display_response_coordinate import DisplayResponseCoordinate
        from ..models.map_reference_section import MapReferenceSection
        from ..models.map_view import MapView
        from ..models.related_result_address import RelatedResultAddress

        d = dict(src_dict)
        relationship = RelatedAddressRelationship(d.pop("relationship"))

        id = d.pop("id")

        title = d.pop("title", UNSET)

        _result_type = d.pop("resultType", UNSET)
        result_type: RelatedAddressResultType | Unset
        if isinstance(_result_type, Unset):
            result_type = UNSET
        else:
            result_type = RelatedAddressResultType(_result_type)

        _house_number_type = d.pop("houseNumberType", UNSET)
        house_number_type: RelatedAddressHouseNumberType | Unset
        if isinstance(_house_number_type, Unset):
            house_number_type = UNSET
        else:
            house_number_type = RelatedAddressHouseNumberType(_house_number_type)

        _address = d.pop("address", UNSET)
        address: RelatedResultAddress | Unset
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = RelatedResultAddress.from_dict(_address)

        _position = d.pop("position", UNSET)
        position: DisplayResponseCoordinate | Unset
        if isinstance(_position, Unset):
            position = UNSET
        else:
            position = DisplayResponseCoordinate.from_dict(_position)

        _access = d.pop("access", UNSET)
        access: list[BasicAccessPoint] | Unset = UNSET
        if _access is not UNSET:
            access = []
            for access_item_data in _access:
                access_item = BasicAccessPoint.from_dict(access_item_data)

                access.append(access_item)

        distance = d.pop("distance", UNSET)

        route_distance = d.pop("routeDistance", UNSET)

        bearing = d.pop("bearing", UNSET)

        _map_view = d.pop("mapView", UNSET)
        map_view: MapView | Unset
        if isinstance(_map_view, Unset):
            map_view = UNSET
        else:
            map_view = MapView.from_dict(_map_view)

        _map_references = d.pop("mapReferences", UNSET)
        map_references: MapReferenceSection | Unset
        if isinstance(_map_references, Unset):
            map_references = UNSET
        else:
            map_references = MapReferenceSection.from_dict(_map_references)

        related_address = cls(
            relationship=relationship,
            id=id,
            title=title,
            result_type=result_type,
            house_number_type=house_number_type,
            address=address,
            position=position,
            access=access,
            distance=distance,
            route_distance=route_distance,
            bearing=bearing,
            map_view=map_view,
            map_references=map_references,
        )

        related_address.additional_properties = d
        return related_address

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.hazardous_goods_restriction_any import HazardousGoodsRestrictionAny
from ..models.truck_type import TruckType
from ..models.tunnel_category import TunnelCategory
from ..models.vehicle_restriction_type import VehicleRestrictionType
from ..models.vehicle_type import VehicleType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.axle_group_weight import AxleGroupWeight
    from ..models.trailer_count_range import TrailerCountRange
    from ..models.truck_axle_count_range import TruckAxleCountRange
    from ..models.vehicle_restriction_max_weight import VehicleRestrictionMaxWeight


T = TypeVar("T", bound="VehicleRestriction")


@_attrs_define
class VehicleRestriction:
    """Provides details about the violated restriction and the violated restriction
    conditions that were evaluated against parameter values set in the routing
    request (such as vehicle properties).

    The restriction is violated only if all of the conditions present are met.

        Example:
            {'$ref': '#/components/examples/restrictionExample'}

        Attributes:
            type_ (VehicleRestrictionType): Detail type. Each type of detail might contain extra attributes.

                **NOTE:** The list of possible detail types may be extended in the future.
                The client application is expected to handle such a case gracefully.
            title (str | Unset): Detail title
            cause (str | Unset): Cause of the notice
            forbidden_hazardous_goods (HazardousGoodsRestrictionAny | list[str] | Unset):
            max_weight (VehicleRestrictionMaxWeight | Unset): Contains the maximum permitted weight, specified in kilograms,
                along with the specific type of the maximum permitted weight restriction.
            max_gross_weight (int | Unset): Contains the maximum permitted weight, specified in kilograms.
                This condition is met when the vehicle's weight exceeds the specified value.

                **NOTE:** This attribute is deprecated, use `maxWeight` instead. It is redundant and is present because of
                backward compatibility reasons.
            max_weight_per_axle (int | Unset): Contains the maximum permitted weight per axle, specified in kilograms.

                This condition is met when the vehicle's `weightPerAxle` exceeds the specified value.
            max_axle_group_weight (AxleGroupWeight | Unset): Contains the maximum allowed weight for an axle-group.
            max_height (int | Unset): Contains the maximum permitted height, specified in centimeters.

                This condition is met when the vehicle's `height` exceeds the specified value.
            max_width (int | Unset): Contains the maximum permitted width, specified in centimeters.

                This condition is met when the vehicle's `width` exceeds the specified value.
            max_length (int | Unset): Contains the maximum permitted length, specified in centimeters.

                This condition is met when the vehicle's `length` exceeds the specified value.
            max_tires_count (int | Unset): Contains the maximum number of permitted tires.

                This condition is met when the vehicle's `tiresCount` exceeds the specified value.
            axle_count (TruckAxleCountRange | Unset): Constrains the restriction to trucks with the number of axles within
                the specified range.
            tunnel_category (TunnelCategory | Unset): Specifies the tunnel category used to restrict the transport of
                specific goods.
            time_dependent (bool | Unset): Indicates that the restriction depends on time.
            truck_type (TruckType | Unset): Specifies the type of the truck

                * `Straight`: A truck on a single frame with a permanently attached cargo area.
                * `Tractor`: A towing vehicle that can pull one or more semi-trailers (also known as a semi-truck).
            vehicle_type (VehicleType | Unset): Specifies the type of the vehicle

                * `StraightTruck`: A truck on a single frame with a permanently attached cargo area. **Note:**
                default value when truck routing mode is used.
                * `Tractor`: A towing vehicle that can pull one or more semi-trailers (also known as a semi-truck).

                **Limitations:** only valid for `transportMode=truck`.
            trailer_count (TrailerCountRange | Unset): Constrains the restriction to vehicles with the number of trailers
                within the specified range.
            max_engine_size_cc (int | Unset): Contains the maximum permitted size of the engine, specified in cubic
                centimeters.

                This condition is met when the vehicle's `engineSizeCC` exceeds the specified value.
            min_engine_size_cc (int | Unset): Contains the minimum permitted size of the engine, specified in cubic
                centimeters.

                This condition is met when the vehicle's `engineSizeCC' is less than the specified value.
            max_occupancy (int | Unset): Contains the maximum permitted occupancy.

                This condition is met when the vehicle's `occupancy` exceeds the specified value.
            min_occupancy (int | Unset): Contains the minimum permitted occupancy.

                This condition is met when the vehicle's `occupancy` is less than the specified value.
            restricted_times (str | Unset): Specifies date and time period during which the restriction applies. Value is a
                string in the Time
                Domain format. Time Domain is part of the GDF (Geographic Data Files) specification, which is an ISO standard.
                Current standard is GDF 5.1 which is [ISO 20524-1:2020](https://www.iso.org/standard/68244.html).

                For a detailed description of the Time Domain specification and usage in routing services, please refer to
                the documentation available in the [Time Domain](https://www.here.com/docs/bundle/routing-api-developer-
                guide-v8/page/concepts/time-domain.html) page of the Developer Guide.
                 Example: -(d1){w1}(d3){d1}.
            max_kpra_length (int | Unset): Contains max permitted kingpin to rear axle length, in centimeters.

                This condition is met when the vehicle's `kpraLength` exceeds this value.
            max_payload_capacity (int | Unset): Contains the maximum allowed payload capacity, specified in kilograms.

                This condition is met when the vehicle's `payloadCapacity` exceeds the specified value.
            unconditional (bool | Unset): Restriction applies unconditionally. Default: False.
    """

    type_: VehicleRestrictionType
    title: str | Unset = UNSET
    cause: str | Unset = UNSET
    forbidden_hazardous_goods: HazardousGoodsRestrictionAny | list[str] | Unset = UNSET
    max_weight: VehicleRestrictionMaxWeight | Unset = UNSET
    max_gross_weight: int | Unset = UNSET
    max_weight_per_axle: int | Unset = UNSET
    max_axle_group_weight: AxleGroupWeight | Unset = UNSET
    max_height: int | Unset = UNSET
    max_width: int | Unset = UNSET
    max_length: int | Unset = UNSET
    max_tires_count: int | Unset = UNSET
    axle_count: TruckAxleCountRange | Unset = UNSET
    tunnel_category: TunnelCategory | Unset = UNSET
    time_dependent: bool | Unset = UNSET
    truck_type: TruckType | Unset = UNSET
    vehicle_type: VehicleType | Unset = UNSET
    trailer_count: TrailerCountRange | Unset = UNSET
    max_engine_size_cc: int | Unset = UNSET
    min_engine_size_cc: int | Unset = UNSET
    max_occupancy: int | Unset = UNSET
    min_occupancy: int | Unset = UNSET
    restricted_times: str | Unset = UNSET
    max_kpra_length: int | Unset = UNSET
    max_payload_capacity: int | Unset = UNSET
    unconditional: bool | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        title = self.title

        cause = self.cause

        forbidden_hazardous_goods: list[str] | str | Unset
        if isinstance(self.forbidden_hazardous_goods, Unset):
            forbidden_hazardous_goods = UNSET
        elif isinstance(self.forbidden_hazardous_goods, HazardousGoodsRestrictionAny):
            forbidden_hazardous_goods = self.forbidden_hazardous_goods.value
        else:
            forbidden_hazardous_goods = self.forbidden_hazardous_goods

        max_weight: dict[str, Any] | Unset = UNSET
        if not isinstance(self.max_weight, Unset):
            max_weight = self.max_weight.to_dict()

        max_gross_weight = self.max_gross_weight

        max_weight_per_axle = self.max_weight_per_axle

        max_axle_group_weight: dict[str, Any] | Unset = UNSET
        if not isinstance(self.max_axle_group_weight, Unset):
            max_axle_group_weight = self.max_axle_group_weight.to_dict()

        max_height = self.max_height

        max_width = self.max_width

        max_length = self.max_length

        max_tires_count = self.max_tires_count

        axle_count: dict[str, Any] | Unset = UNSET
        if not isinstance(self.axle_count, Unset):
            axle_count = self.axle_count.to_dict()

        tunnel_category: str | Unset = UNSET
        if not isinstance(self.tunnel_category, Unset):
            tunnel_category = self.tunnel_category.value

        time_dependent = self.time_dependent

        truck_type: str | Unset = UNSET
        if not isinstance(self.truck_type, Unset):
            truck_type = self.truck_type.value

        vehicle_type: str | Unset = UNSET
        if not isinstance(self.vehicle_type, Unset):
            vehicle_type = self.vehicle_type.value

        trailer_count: dict[str, Any] | Unset = UNSET
        if not isinstance(self.trailer_count, Unset):
            trailer_count = self.trailer_count.to_dict()

        max_engine_size_cc = self.max_engine_size_cc

        min_engine_size_cc = self.min_engine_size_cc

        max_occupancy = self.max_occupancy

        min_occupancy = self.min_occupancy

        restricted_times = self.restricted_times

        max_kpra_length = self.max_kpra_length

        max_payload_capacity = self.max_payload_capacity

        unconditional = self.unconditional

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
        })
        if title is not UNSET:
            field_dict["title"] = title
        if cause is not UNSET:
            field_dict["cause"] = cause
        if forbidden_hazardous_goods is not UNSET:
            field_dict["forbiddenHazardousGoods"] = forbidden_hazardous_goods
        if max_weight is not UNSET:
            field_dict["maxWeight"] = max_weight
        if max_gross_weight is not UNSET:
            field_dict["maxGrossWeight"] = max_gross_weight
        if max_weight_per_axle is not UNSET:
            field_dict["maxWeightPerAxle"] = max_weight_per_axle
        if max_axle_group_weight is not UNSET:
            field_dict["maxAxleGroupWeight"] = max_axle_group_weight
        if max_height is not UNSET:
            field_dict["maxHeight"] = max_height
        if max_width is not UNSET:
            field_dict["maxWidth"] = max_width
        if max_length is not UNSET:
            field_dict["maxLength"] = max_length
        if max_tires_count is not UNSET:
            field_dict["maxTiresCount"] = max_tires_count
        if axle_count is not UNSET:
            field_dict["axleCount"] = axle_count
        if tunnel_category is not UNSET:
            field_dict["tunnelCategory"] = tunnel_category
        if time_dependent is not UNSET:
            field_dict["timeDependent"] = time_dependent
        if truck_type is not UNSET:
            field_dict["truckType"] = truck_type
        if vehicle_type is not UNSET:
            field_dict["vehicleType"] = vehicle_type
        if trailer_count is not UNSET:
            field_dict["trailerCount"] = trailer_count
        if max_engine_size_cc is not UNSET:
            field_dict["maxEngineSizeCC"] = max_engine_size_cc
        if min_engine_size_cc is not UNSET:
            field_dict["minEngineSizeCC"] = min_engine_size_cc
        if max_occupancy is not UNSET:
            field_dict["maxOccupancy"] = max_occupancy
        if min_occupancy is not UNSET:
            field_dict["minOccupancy"] = min_occupancy
        if restricted_times is not UNSET:
            field_dict["restrictedTimes"] = restricted_times
        if max_kpra_length is not UNSET:
            field_dict["maxKpraLength"] = max_kpra_length
        if max_payload_capacity is not UNSET:
            field_dict["maxPayloadCapacity"] = max_payload_capacity
        if unconditional is not UNSET:
            field_dict["unconditional"] = unconditional

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.axle_group_weight import AxleGroupWeight
        from ..models.trailer_count_range import TrailerCountRange
        from ..models.truck_axle_count_range import TruckAxleCountRange
        from ..models.vehicle_restriction_max_weight import VehicleRestrictionMaxWeight

        d = dict(src_dict)
        type_ = VehicleRestrictionType(d.pop("type"))

        title = d.pop("title", UNSET)

        cause = d.pop("cause", UNSET)

        def _parse_forbidden_hazardous_goods(
            data: object,
        ) -> HazardousGoodsRestrictionAny | list[str] | Unset:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                componentsschemas_forbidden_hazardous_goods_type_0 = (
                    HazardousGoodsRestrictionAny(data)
                )

                return componentsschemas_forbidden_hazardous_goods_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, list):
                raise TypeError()
            componentsschemas_forbidden_hazardous_goods_type_1 = cast(list[str], data)

            return componentsschemas_forbidden_hazardous_goods_type_1

        forbidden_hazardous_goods = _parse_forbidden_hazardous_goods(
            d.pop("forbiddenHazardousGoods", UNSET)
        )

        _max_weight = d.pop("maxWeight", UNSET)
        max_weight: VehicleRestrictionMaxWeight | Unset
        if isinstance(_max_weight, Unset):
            max_weight = UNSET
        else:
            max_weight = VehicleRestrictionMaxWeight.from_dict(_max_weight)

        max_gross_weight = d.pop("maxGrossWeight", UNSET)

        max_weight_per_axle = d.pop("maxWeightPerAxle", UNSET)

        _max_axle_group_weight = d.pop("maxAxleGroupWeight", UNSET)
        max_axle_group_weight: AxleGroupWeight | Unset
        if isinstance(_max_axle_group_weight, Unset):
            max_axle_group_weight = UNSET
        else:
            max_axle_group_weight = AxleGroupWeight.from_dict(_max_axle_group_weight)

        max_height = d.pop("maxHeight", UNSET)

        max_width = d.pop("maxWidth", UNSET)

        max_length = d.pop("maxLength", UNSET)

        max_tires_count = d.pop("maxTiresCount", UNSET)

        _axle_count = d.pop("axleCount", UNSET)
        axle_count: TruckAxleCountRange | Unset
        if isinstance(_axle_count, Unset):
            axle_count = UNSET
        else:
            axle_count = TruckAxleCountRange.from_dict(_axle_count)

        _tunnel_category = d.pop("tunnelCategory", UNSET)
        tunnel_category: TunnelCategory | Unset
        if isinstance(_tunnel_category, Unset):
            tunnel_category = UNSET
        else:
            tunnel_category = TunnelCategory(_tunnel_category)

        time_dependent = d.pop("timeDependent", UNSET)

        _truck_type = d.pop("truckType", UNSET)
        truck_type: TruckType | Unset
        if isinstance(_truck_type, Unset):
            truck_type = UNSET
        else:
            truck_type = TruckType(_truck_type)

        _vehicle_type = d.pop("vehicleType", UNSET)
        vehicle_type: VehicleType | Unset
        if isinstance(_vehicle_type, Unset):
            vehicle_type = UNSET
        else:
            vehicle_type = VehicleType(_vehicle_type)

        _trailer_count = d.pop("trailerCount", UNSET)
        trailer_count: TrailerCountRange | Unset
        if isinstance(_trailer_count, Unset):
            trailer_count = UNSET
        else:
            trailer_count = TrailerCountRange.from_dict(_trailer_count)

        max_engine_size_cc = d.pop("maxEngineSizeCC", UNSET)

        min_engine_size_cc = d.pop("minEngineSizeCC", UNSET)

        max_occupancy = d.pop("maxOccupancy", UNSET)

        min_occupancy = d.pop("minOccupancy", UNSET)

        restricted_times = d.pop("restrictedTimes", UNSET)

        max_kpra_length = d.pop("maxKpraLength", UNSET)

        max_payload_capacity = d.pop("maxPayloadCapacity", UNSET)

        unconditional = d.pop("unconditional", UNSET)

        vehicle_restriction = cls(
            type_=type_,
            title=title,
            cause=cause,
            forbidden_hazardous_goods=forbidden_hazardous_goods,
            max_weight=max_weight,
            max_gross_weight=max_gross_weight,
            max_weight_per_axle=max_weight_per_axle,
            max_axle_group_weight=max_axle_group_weight,
            max_height=max_height,
            max_width=max_width,
            max_length=max_length,
            max_tires_count=max_tires_count,
            axle_count=axle_count,
            tunnel_category=tunnel_category,
            time_dependent=time_dependent,
            truck_type=truck_type,
            vehicle_type=vehicle_type,
            trailer_count=trailer_count,
            max_engine_size_cc=max_engine_size_cc,
            min_engine_size_cc=min_engine_size_cc,
            max_occupancy=max_occupancy,
            min_occupancy=min_occupancy,
            restricted_times=restricted_times,
            max_kpra_length=max_kpra_length,
            max_payload_capacity=max_payload_capacity,
            unconditional=unconditional,
        )

        vehicle_restriction.additional_properties = d
        return vehicle_restriction

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="EVPost")


@_attrs_define
class EVPost:
    """
    Attributes:
        preferred_brands (list[str] | Unset): An array of charging station brand IDs. If `makeReachable` is set to
            `true`, then charging stations from the specified brands
            will be preferred as potential charging stops, even if this makes the total route duration (travel time plus
            charging time) longer (up to 15 minutes per charging stop)

            Example of a parameter value preferring two charging station brands:
            ```
              {
                "preferredBrands": [
                  "6e1a148e8ddf06f613599134197b7c1c",
                  "6211c90a063d36429b599dda79ae85e3"
                ]
              }
              ```
        preferred_charge_point_operators (list[str] | Unset): An array of charge-point-operator identifiers. If
            `makeReachable` is set to `true`, then charging stations with one
            of the listed charge-point-operator ids will be preferred as potential charging stops, even if this makes the
            total
            route duration (travel time plus charging time) longer (up to 15 minutes per charging stop)

            Example with two preferred charge-point-operators specified:
              ```
              {
                "preferredChargePointOperators": [
                  "1f608648-cca5-11ed-bb1e-42010aa40002",
                  "1f90fdf0-cca5-11ed-be23-42010aa40002"
                ]
              }
              ```
    """

    preferred_brands: list[str] | Unset = UNSET
    preferred_charge_point_operators: list[str] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        preferred_brands: list[str] | Unset = UNSET
        if not isinstance(self.preferred_brands, Unset):
            preferred_brands = self.preferred_brands

        preferred_charge_point_operators: list[str] | Unset = UNSET
        if not isinstance(self.preferred_charge_point_operators, Unset):
            preferred_charge_point_operators = self.preferred_charge_point_operators

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if preferred_brands is not UNSET:
            field_dict["preferredBrands"] = preferred_brands
        if preferred_charge_point_operators is not UNSET:
            field_dict["preferredChargePointOperators"] = (
                preferred_charge_point_operators
            )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        preferred_brands = cast(list[str], d.pop("preferredBrands", UNSET))

        preferred_charge_point_operators = cast(
            list[str], d.pop("preferredChargePointOperators", UNSET)
        )

        ev_post = cls(
            preferred_brands=preferred_brands,
            preferred_charge_point_operators=preferred_charge_point_operators,
        )

        ev_post.additional_properties = d
        return ev_post

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.coordinate import Coordinate


T = TypeVar("T", bound="PolygonArea")


@_attrs_define
class PolygonArea:
    """A polygon defined as a list of coordinates.

    The polygon is automatically closed, so repeating the first vertex is not required. Self-intersecting polygons are
    not supported.

        Attributes:
            type_ (str):  Example: polygon.
            outer (list[Coordinate]): List of coordinates defining the outline of the polygon.
    """

    type_: str
    outer: list[Coordinate]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        outer = []
        for outer_item_data in self.outer:
            outer_item = outer_item_data.to_dict()
            outer.append(outer_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "outer": outer,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.coordinate import Coordinate

        d = dict(src_dict)
        type_ = d.pop("type")

        outer = []
        _outer = d.pop("outer")
        for outer_item_data in _outer:
            outer_item = Coordinate.from_dict(outer_item_data)

            outer.append(outer_item)

        polygon_area = cls(
            type_=type_,
            outer=outer,
        )

        polygon_area.additional_properties = d
        return polygon_area

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="BaseAction")


@_attrs_define
class BaseAction:
    """An action to be performed between sections or during a section.

    Attributes:
        action (str): The type of the action.

            **NOTE:** The list of possible actions may be extended in the future. The client application should handle such
            a case gracefully.
        duration (int): Duration in seconds. Example: 198.
        instruction (str | Unset): Description of the action (e.g. Turn left onto Minna St.).
    """

    action: str
    duration: int
    instruction: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        action = self.action

        duration = self.duration

        instruction = self.instruction

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "action": action,
            "duration": duration,
        })
        if instruction is not UNSET:
            field_dict["instruction"] = instruction

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        action = d.pop("action")

        duration = d.pop("duration")

        instruction = d.pop("instruction", UNSET)

        base_action = cls(
            action=action,
            duration=duration,
            instruction=instruction,
        )

        base_action.additional_properties = d
        return base_action

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.category_ref import CategoryRef
    from ..models.structured_opening_hours import StructuredOpeningHours


T = TypeVar("T", bound="OpeningHours")


@_attrs_define
class OpeningHours:
    """
    Attributes:
        text (list[str]):
        structured (list[StructuredOpeningHours]): List of iCalender-based structured representations of opening hours
        categories (list[CategoryRef] | Unset): The list of place categories, this set of opening hours refers to.
        is_open (bool | Unset):
    """

    text: list[str]
    structured: list[StructuredOpeningHours]
    categories: list[CategoryRef] | Unset = UNSET
    is_open: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        text = self.text

        structured = []
        for structured_item_data in self.structured:
            structured_item = structured_item_data.to_dict()
            structured.append(structured_item)

        categories: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.categories, Unset):
            categories = []
            for categories_item_data in self.categories:
                categories_item = categories_item_data.to_dict()
                categories.append(categories_item)

        is_open = self.is_open

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "text": text,
            "structured": structured,
        })
        if categories is not UNSET:
            field_dict["categories"] = categories
        if is_open is not UNSET:
            field_dict["isOpen"] = is_open

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.category_ref import CategoryRef
        from ..models.structured_opening_hours import StructuredOpeningHours

        d = dict(src_dict)
        text = cast(list[str], d.pop("text"))

        structured = []
        _structured = d.pop("structured")
        for structured_item_data in _structured:
            structured_item = StructuredOpeningHours.from_dict(structured_item_data)

            structured.append(structured_item)

        _categories = d.pop("categories", UNSET)
        categories: list[CategoryRef] | Unset = UNSET
        if _categories is not UNSET:
            categories = []
            for categories_item_data in _categories:
                categories_item = CategoryRef.from_dict(categories_item_data)

                categories.append(categories_item)

        is_open = d.pop("isOpen", UNSET)

        opening_hours = cls(
            text=text,
            structured=structured,
            categories=categories,
            is_open=is_open,
        )

        opening_hours.additional_properties = d
        return opening_hours

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.admin_names import AdminNames


T = TypeVar("T", bound="TranslationsGeocode")


@_attrs_define
class TranslationsGeocode:
    """
    Attributes:
        state_names (list[AdminNames] | Unset): The list of all state names and translations applicable to an address
            field, grouped by alternative views on it. For example, if subdivision of a country from administrative and
            postal views is different, some administrative levels may have two groups of names: from administrative and from
            postal views.
        county_names (list[AdminNames] | Unset): The list of all county names and translations applicable to an address
            field, grouped by alternative views on it. For example, if subdivision of a country from administrative and
            postal views is different, some administrative levels may have two groups of names: from administrative and from
            postal views.
        city_names (list[AdminNames] | Unset): The list of all city names and translations applicable to an address
            field, grouped by alternative views on it. For example, if subdivision of a country from administrative and
            postal views is different, some administrative levels may have two groups of names: from administrative and from
            postal views.
        district_names (list[AdminNames] | Unset): The list of all district names and translations applicable to an
            address field, grouped by alternative views on it. For example, if subdivision of a country from administrative
            and postal views is different, some administrative levels may have two groups of names: from administrative and
            from postal views.
    """

    state_names: list[AdminNames] | Unset = UNSET
    county_names: list[AdminNames] | Unset = UNSET
    city_names: list[AdminNames] | Unset = UNSET
    district_names: list[AdminNames] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        state_names: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.state_names, Unset):
            state_names = []
            for state_names_item_data in self.state_names:
                state_names_item = state_names_item_data.to_dict()
                state_names.append(state_names_item)

        county_names: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.county_names, Unset):
            county_names = []
            for county_names_item_data in self.county_names:
                county_names_item = county_names_item_data.to_dict()
                county_names.append(county_names_item)

        city_names: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.city_names, Unset):
            city_names = []
            for city_names_item_data in self.city_names:
                city_names_item = city_names_item_data.to_dict()
                city_names.append(city_names_item)

        district_names: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.district_names, Unset):
            district_names = []
            for district_names_item_data in self.district_names:
                district_names_item = district_names_item_data.to_dict()
                district_names.append(district_names_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if state_names is not UNSET:
            field_dict["stateNames"] = state_names
        if county_names is not UNSET:
            field_dict["countyNames"] = county_names
        if city_names is not UNSET:
            field_dict["cityNames"] = city_names
        if district_names is not UNSET:
            field_dict["districtNames"] = district_names

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.admin_names import AdminNames

        d = dict(src_dict)
        _state_names = d.pop("stateNames", UNSET)
        state_names: list[AdminNames] | Unset = UNSET
        if _state_names is not UNSET:
            state_names = []
            for state_names_item_data in _state_names:
                state_names_item = AdminNames.from_dict(state_names_item_data)

                state_names.append(state_names_item)

        _county_names = d.pop("countyNames", UNSET)
        county_names: list[AdminNames] | Unset = UNSET
        if _county_names is not UNSET:
            county_names = []
            for county_names_item_data in _county_names:
                county_names_item = AdminNames.from_dict(county_names_item_data)

                county_names.append(county_names_item)

        _city_names = d.pop("cityNames", UNSET)
        city_names: list[AdminNames] | Unset = UNSET
        if _city_names is not UNSET:
            city_names = []
            for city_names_item_data in _city_names:
                city_names_item = AdminNames.from_dict(city_names_item_data)

                city_names.append(city_names_item)

        _district_names = d.pop("districtNames", UNSET)
        district_names: list[AdminNames] | Unset = UNSET
        if _district_names is not UNSET:
            district_names = []
            for district_names_item_data in _district_names:
                district_names_item = AdminNames.from_dict(district_names_item_data)

                district_names.append(district_names_item)

        translations_geocode = cls(
            state_names=state_names,
            county_names=county_names,
            city_names=city_names,
            district_names=district_names,
        )

        translations_geocode.additional_properties = d
        return translations_geocode

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Physical")


@_attrs_define
class Physical:
    """
    Attributes:
        boat_ferry (bool | Unset): The segment represents a generalised route of a boat ferry for passengers or vehicles
            over water,
            including routes to pedestrian only islands, tourist areas, commuter ferry routes open to only pedestrians, etc.

            - `true`: The road is a boat ferry.
            - `false`: The road isn't a boat ferry.
        bridge (bool | Unset): Identifies a structure that allows a road, railway, or walkway to pass over another road,
            railway, waterway, or valley serving map display and route guidance functionalities.

            - `true`: The road is a bridge.
            - `false`: The road isn't a bridge.
        delivery_road (bool | Unset): Indication of a delivery road.

            - `true`: The physical characteristic of the road is that it functions as a delivery road.
            - `false`: The physical characteristic of the road is that it doesn't function as a delivery road.
        movable_bridge (bool | Unset): Movable Bridge indicates a bridge that moves to allow passage (usually) for boats
            or barges.

            - `true`: The road is a movable bridge.
            - `false`: The road isn't a movable bridge.
        multiply_digitized (bool | Unset): Multiply Digitised identifies separately digitised roads, i.e., roads that
            are digitised with one line per
            direction of traffic instead of one line per road.

            - `true`: The physical characteristic of the road is that it functions as multiplyDigitized.
            - `false`: The physical characteristic of the road is that it doesn't function as multiplyDigitized.
        paved (bool | Unset): Indicates whether the navigable segment is paved.

            - `true`: The physical characteristic of the road is paved.
            - `false`: The physical characteristic of the road isn't paved.
        private (bool | Unset): Private identifies roads that are not maintained by an organisation responsible for
            maintenance of
            public roads.

            - `true`: The road is private..
            - `false`: The road isn't private.
        rail_ferry (bool | Unset): The segment represents a generalised route of a ferry for passengers or vehicles via
            rail.

            - `true`: The road is a rail flerry.
            - `false`: The road isn't a rail flerry.
        tunnel (bool | Unset): Identifies an enclosed (on all sides) passageway through or under an obstruction.

            - `true`: The road is a tunnel.
            - `false`: The road isn't a tunnel.
    """

    boat_ferry: bool | Unset = UNSET
    bridge: bool | Unset = UNSET
    delivery_road: bool | Unset = UNSET
    movable_bridge: bool | Unset = UNSET
    multiply_digitized: bool | Unset = UNSET
    paved: bool | Unset = UNSET
    private: bool | Unset = UNSET
    rail_ferry: bool | Unset = UNSET
    tunnel: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        boat_ferry = self.boat_ferry

        bridge = self.bridge

        delivery_road = self.delivery_road

        movable_bridge = self.movable_bridge

        multiply_digitized = self.multiply_digitized

        paved = self.paved

        private = self.private

        rail_ferry = self.rail_ferry

        tunnel = self.tunnel

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if boat_ferry is not UNSET:
            field_dict["boatFerry"] = boat_ferry
        if bridge is not UNSET:
            field_dict["bridge"] = bridge
        if delivery_road is not UNSET:
            field_dict["deliveryRoad"] = delivery_road
        if movable_bridge is not UNSET:
            field_dict["movableBridge"] = movable_bridge
        if multiply_digitized is not UNSET:
            field_dict["multiplyDigitized"] = multiply_digitized
        if paved is not UNSET:
            field_dict["paved"] = paved
        if private is not UNSET:
            field_dict["private"] = private
        if rail_ferry is not UNSET:
            field_dict["railFerry"] = rail_ferry
        if tunnel is not UNSET:
            field_dict["tunnel"] = tunnel

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        boat_ferry = d.pop("boatFerry", UNSET)

        bridge = d.pop("bridge", UNSET)

        delivery_road = d.pop("deliveryRoad", UNSET)

        movable_bridge = d.pop("movableBridge", UNSET)

        multiply_digitized = d.pop("multiplyDigitized", UNSET)

        paved = d.pop("paved", UNSET)

        private = d.pop("private", UNSET)

        rail_ferry = d.pop("railFerry", UNSET)

        tunnel = d.pop("tunnel", UNSET)

        physical = cls(
            boat_ferry=boat_ferry,
            bridge=bridge,
            delivery_road=delivery_road,
            movable_bridge=movable_bridge,
            multiply_digitized=multiply_digitized,
            paved=paved,
            private=private,
            rail_ferry=rail_ferry,
            tunnel=tunnel,
        )

        physical.additional_properties = d
        return physical

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class MatchInfoQq(str, Enum):
    CITY = "city"
    COUNTRY = "country"
    COUNTY = "county"
    DISTRICT = "district"
    HOUSENUMBER = "houseNumber"
    POSTALCODE = "postalCode"
    STATE = "state"
    STREET = "street"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.car_fuel import CarFuel
    from ..models.truck_fuel import TruckFuel


T = TypeVar("T", bound="FuelStationAttributes")


@_attrs_define
class FuelStationAttributes:
    """
    Attributes:
        fuel_types (list[CarFuel | TruckFuel] | Unset): **BETA, RESTRICTED**

            List of fuel types available or not available at this place
        pay_at_the_pump (bool | Unset): **BETA, RESTRICTED**

            When set to:
            - `true` indicates that payment can be made at fuel pumps via credit card or other method at this place
            - `false` indicates that payment can not be made at fuel pumps via credit card or other method at this place
        high_volume_pumps (bool | Unset): **BETA, RESTRICTED**

            When set to:
            - `true` indicates that high volume pumps for commercial vehicles are available at this place
            - `false` indicates that high volume pumps for commercial vehicles are not available at this place
    """

    fuel_types: list[CarFuel | TruckFuel] | Unset = UNSET
    pay_at_the_pump: bool | Unset = UNSET
    high_volume_pumps: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.car_fuel import CarFuel

        fuel_types: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.fuel_types, Unset):
            fuel_types = []
            for fuel_types_item_data in self.fuel_types:
                fuel_types_item: dict[str, Any]
                if isinstance(fuel_types_item_data, CarFuel):
                    fuel_types_item = fuel_types_item_data.to_dict()
                else:
                    fuel_types_item = fuel_types_item_data.to_dict()

                fuel_types.append(fuel_types_item)

        pay_at_the_pump = self.pay_at_the_pump

        high_volume_pumps = self.high_volume_pumps

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if fuel_types is not UNSET:
            field_dict["fuelTypes"] = fuel_types
        if pay_at_the_pump is not UNSET:
            field_dict["payAtThePump"] = pay_at_the_pump
        if high_volume_pumps is not UNSET:
            field_dict["highVolumePumps"] = high_volume_pumps

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.car_fuel import CarFuel
        from ..models.truck_fuel import TruckFuel

        d = dict(src_dict)
        _fuel_types = d.pop("fuelTypes", UNSET)
        fuel_types: list[CarFuel | TruckFuel] | Unset = UNSET
        if _fuel_types is not UNSET:
            fuel_types = []
            for fuel_types_item_data in _fuel_types:

                def _parse_fuel_types_item(data: object) -> CarFuel | TruckFuel:
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        fuel_types_item_type_0 = CarFuel.from_dict(data)

                        return fuel_types_item_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    fuel_types_item_type_1 = TruckFuel.from_dict(data)

                    return fuel_types_item_type_1

                fuel_types_item = _parse_fuel_types_item(fuel_types_item_data)

                fuel_types.append(fuel_types_item)

        pay_at_the_pump = d.pop("payAtThePump", UNSET)

        high_volume_pumps = d.pop("highVolumePumps", UNSET)

        fuel_station_attributes = cls(
            fuel_types=fuel_types,
            pay_at_the_pump=pay_at_the_pump,
            high_volume_pumps=high_volume_pumps,
        )

        fuel_station_attributes.additional_properties = d
        return fuel_station_attributes

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
